/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 8; tab-width: 8 -*-
 *
 * nautilus-burn-drive.c: easy to use cd burner software
 *
 * Copyright (C) 2002-2004 Red Hat, Inc.
 * Copyright (C) 2005 William Jon McCann <mccann@jhu.edu>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 * 
 * Authors: Alexander Larsson <alexl@redhat.com>
 *          Bastien Nocera <hadess@hadess.net>
 *          William Jon McCann <mccann@jhu.edu>
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <math.h>
#include <fcntl.h>
#include <unistd.h>
#include <string.h>

#include <glib.h>
#include <glib/gstdio.h>

#include "nautilus-burn-drive.h"
#include "nautilus-burn-drive-common.h"

#ifdef __linux__
#include "nautilus-burn-drive-linux.h"
#endif

#ifdef __FreeBSD__
#include "nautilus-burn-drive-freebsd.h"
#endif

int get_dvd_r_rw_profile (const char *name);
int get_read_write_speed (int        fd, 
                          int        *read_speed, 
                          int        *write_speed);

#ifdef __FreeBSD__

gpointer
open_ioctl_handle (const char *device)
{
  struct cam_device *cam;

  cam = cam_open_device (device, O_RDWR);

  return (cam ? (gpointer) cam : INVALID_HANDLE);
}

void
close_ioctl_handle (gpointer handle)
{
  cam_close_device ((struct cam_device *) handle);
}

#else

gpointer
open_ioctl_handle (const char *device)
{
  int fd;

  if ((fd = g_open (device, O_RDWR | O_EXCL | O_NONBLOCK, 0)) < 0
      && (fd = g_open (device, O_RDONLY | O_EXCL | O_NONBLOCK, 0)) < 0)
    return INVALID_HANDLE;

  return GINT_TO_POINTER (fd);
}

void
close_ioctl_handle (gpointer handle)
{
  close (GPOINTER_TO_INT (handle));
}

#endif

int
get_device_max_write_speed (char *device)
{
  gpointer ioctl_handle;
  int fd;
  int max_speed;
  int read_speed, write_speed;

  max_speed = -1;

  ioctl_handle = open_ioctl_handle (device);
  if (ioctl_handle == INVALID_HANDLE)
    return -1;

  fd = get_ioctl_handle_fd (ioctl_handle);
  get_read_write_speed (fd, &read_speed, &write_speed);
  close_ioctl_handle (ioctl_handle);
  max_speed = (int) floor (write_speed) / CD_ROM_SPEED;

  return max_speed;
}

void
nautilus_burn_drive_add_dvd_plus (NautilusBurnDrive * drive)
{
  int caps;

  caps = get_dvd_r_rw_profile (drive->device);

  if (caps == -1)
    return;

  if (caps == 2)
  {
    drive->type |= NAUTILUS_BURN_DRIVE_TYPE_DVD_PLUS_RW_RECORDER;
    drive->type |= NAUTILUS_BURN_DRIVE_TYPE_DVD_PLUS_R_RECORDER;
  }
  else if (caps == 0)
    drive->type |= NAUTILUS_BURN_DRIVE_TYPE_DVD_PLUS_R_RECORDER;
  else if (caps == 1)
    drive->type |= NAUTILUS_BURN_DRIVE_TYPE_DVD_PLUS_RW_RECORDER;
}

void
nautilus_burn_drive_add_whitelist (NautilusBurnDrive *drive)
{
  NautilusBurnWhiteDrive recorder_whitelist[] =
  {
    { "IOMEGA - CDRW9602EXT-B", TRUE, TRUE,  FALSE, FALSE },
    { "SONY - CD-R CDU948S",    TRUE, FALSE, FALSE, FALSE },
  };

  guint i;

  for (i = 0; i < G_N_ELEMENTS (recorder_whitelist); i++) 
  {
    if (!strcmp (drive->display_name, recorder_whitelist [i].name)) 
    {
      if (recorder_whitelist [i].can_write_cdr)
        drive->type |= NAUTILUS_BURN_DRIVE_TYPE_CD_RECORDER;
      if (recorder_whitelist [i].can_write_cdrw)
        drive->type |= NAUTILUS_BURN_DRIVE_TYPE_CDRW_RECORDER;
      if (recorder_whitelist [i].can_write_dvdr)
        drive->type |= NAUTILUS_BURN_DRIVE_TYPE_DVD_RW_RECORDER;
      if (recorder_whitelist [i].can_write_dvdram)
        drive->type |= NAUTILUS_BURN_DRIVE_TYPE_DVD_RAM_RECORDER;
    }
  }
}

static int
get_device_max_read_speed (char *device)
{
  gpointer ioctl_handle;
  int      fd;
  int      max_speed;
  int      read_speed, write_speed;

  max_speed = -1;

  ioctl_handle = open_ioctl_handle (device);
  if (ioctl_handle == INVALID_HANDLE)
    return -1;

  fd = get_ioctl_handle_fd (ioctl_handle);
  get_read_write_speed (fd, &read_speed, &write_speed);
  close_ioctl_handle (ioctl_handle);
  max_speed = (int)floor (read_speed) / CD_ROM_SPEED;

  return max_speed;
}

static char *
cdrecord_get_stdout_for_id (char *id)
{
  int         max_speed, i;
  const char *argv [20]; /* Shouldn't need more than 20 arguments */
  char       *dev_str, *stdout_data;

  max_speed = -1;

  i = 0;
  argv [i++] = "cdrecord";
  argv [i++] = "-prcap";
  dev_str = g_strdup_printf ("dev=%s", id);
  argv [i++] = dev_str;
  argv [i++] = NULL;

  if (g_spawn_sync (NULL, (char **)argv, NULL, G_SPAWN_SEARCH_PATH | G_SPAWN_STDERR_TO_DEV_NULL, 
        NULL, NULL, &stdout_data, NULL, NULL, NULL)) 
  {
    g_free (dev_str);
    return stdout_data;
  }

  g_free (dev_str);
  return NULL;
}

void
nautilus_burn_drive_get_cd_properties (NautilusBurnDrive *drive)
{
	char *stdout_data, *drive_cap;

  drive->max_speed_write = -1;
  drive->max_speed_read = -1;
  drive->type = 0;

	drive->max_speed_read = get_device_max_read_speed (drive->device);
	drive->max_speed_write = get_device_max_write_speed (drive->device);

	stdout_data = cdrecord_get_stdout_for_id (drive->cdrecord_id);
	if (stdout_data == NULL)
		return;

	drive_cap = strstr (stdout_data, "Does write DVD-RAM media");
	if (drive_cap != NULL)
		drive->type |= NAUTILUS_BURN_DRIVE_TYPE_DVD_RAM_RECORDER;
	drive_cap = strstr (stdout_data, "Does read DVD-R media");
	if (drive_cap != NULL)
		drive->type |= NAUTILUS_BURN_DRIVE_TYPE_DVD_RW_RECORDER;
	drive_cap = strstr (stdout_data, "Does read DVD-ROM media");
	if (drive_cap != NULL)
		drive->type |= NAUTILUS_BURN_DRIVE_TYPE_DVD_DRIVE;
	drive_cap = strstr (stdout_data, "Does write CD-RW media");
	if (drive_cap != NULL)
		drive->type |= NAUTILUS_BURN_DRIVE_TYPE_CDRW_RECORDER;
	drive_cap = strstr (stdout_data, "Does write CD-R media");
	if (drive_cap != NULL)
		drive->type |= NAUTILUS_BURN_DRIVE_TYPE_CD_RECORDER;
	drive_cap = strstr (stdout_data, "Does read CD-R media");
	if (drive_cap != NULL)
		drive->type |= NAUTILUS_BURN_DRIVE_TYPE_CD_DRIVE;

	g_free (stdout_data);
}
static char *
cdrecord_scan_get_stdout (void)
{
  int max_speed, i;
  const char *argv[20];         /* Shouldn't need more than 20 arguments */
  char *stdout_data;

  max_speed = -1;

  i = 0;
  argv[i++] = "cdrecord";
  argv[i++] = "-scanbus";
  argv[i++] = NULL;

  if (g_spawn_sync (NULL, (char **) argv, NULL, G_SPAWN_SEARCH_PATH | G_SPAWN_STDERR_TO_DEV_NULL,
                    NULL, NULL, &stdout_data, NULL, NULL, NULL))
    return stdout_data;

  return NULL;
}

#define DEFAULT_SPEED 2

GList *
cdrecord_scan (gboolean recorder_only)
{
  GList *drives_list;
  NautilusBurnDrive *drive;
  char *stdout_data, **lines, vendor[9], model[17];
  int i, bus, id, lun, index;

  drives_list = NULL;

  stdout_data = cdrecord_scan_get_stdout ();
  if (stdout_data == NULL)
    return drives_list;

  lines = g_strsplit (stdout_data, "\n", 0);
  g_free (stdout_data);

  for (i = 0; lines[i] != NULL; i++)
  {
    if (sscanf (lines[i], "\t%d,%d,%d\t  %d) '%8c' '%16c'", &bus, &id, &lun, &index, vendor, model) != 6)
      continue;

    vendor[8] = '\0';
    model[16] = '\0';

    drive = nautilus_burn_drive_new ();
    drive->display_name = g_strdup_printf ("%s - %s", g_strstrip (vendor), g_strstrip (model));
    drive->cdrecord_id = g_strdup_printf ("%d,%d,%d", bus, id, lun);
    /*
     * FIXME we don't have any way to guess the real device
     * * from the info we get from CDRecord 
     */
    drive->device = g_strdup_printf ("/dev/pg%d", index);
    nautilus_burn_drive_get_cd_properties (drive);
    nautilus_burn_drive_add_whitelist (drive);
    if (drive->type & NAUTILUS_BURN_DRIVE_TYPE_CD_RECORDER || drive->type & NAUTILUS_BURN_DRIVE_TYPE_CDRW_RECORDER
        || drive->type & NAUTILUS_BURN_DRIVE_TYPE_DVD_RAM_RECORDER || drive->type & NAUTILUS_BURN_DRIVE_TYPE_DVD_RW_RECORDER
        || !recorder_only)
    {
      if (drive->max_speed_read == -1)
        drive->max_speed_read = DEFAULT_SPEED;
      if (drive->max_speed_write == -1)
        drive->max_speed_write = DEFAULT_SPEED;

      if (drive->type & NAUTILUS_BURN_DRIVE_TYPE_DVD_DRIVE)
        nautilus_burn_drive_add_dvd_plus (drive);

      drives_list = g_list_append (drives_list, drive);
    }
    else
      nautilus_burn_drive_free (drive);
  }

  g_strfreev (lines);

  return drives_list;
}


