/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 8; tab-width: 8 -*-
 *
 * nautilus-burn-drive.c: easy to use cd burner software
 *
 * Copyright (C) 2002-2004 Red Hat, Inc.
 * Copyright (C) 2005 William Jon McCann <mccann@jhu.edu>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 * 
 * Authors: Alexander Larsson <alexl@redhat.com>
 *          Bastien Nocera <hadess@hadess.net>
 *          William Jon McCann <mccann@jhu.edu>
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef USE_HAL

#include <string.h>
#include <libhal.h>
#include <glib.h>

#include "nautilus-burn-drive.h"
#include "nautilus-burn-drive-common.h"

#ifdef USE_HAL5
#define GET_BOOL_PROP(x) (libhal_device_property_exists (ctx, device_names [i], x, NULL) && libhal_device_get_property_bool (ctx, device_names [i], x, NULL))
#else
#define GET_BOOL_PROP(x) (hal_device_property_exists (ctx, device_names [i], x) && hal_device_get_property_bool (ctx, device_names [i], x))
#endif

static LibHalContext *
get_hal_context (void)
{
  static LibHalContext *ctx = NULL;

#ifdef USE_HAL5
  if (ctx == NULL)
  {
    ctx = libhal_ctx_new ();

    if (!ctx)
      g_warning ("Could not create a HAL context\n");
    else
    {
      DBusError error;
      DBusConnection *dbus_conn;

      dbus_error_init (&error);
      dbus_conn = dbus_bus_get (DBUS_BUS_SYSTEM, &error);

      if (!dbus_conn)
      {
        g_warning ("Could not connect to system bus: %s\n", error.message);

        if (dbus_error_is_set (&error))
          dbus_error_free (&error);

        libhal_ctx_free (ctx);
        ctx = NULL;
      }
      else
      {
        libhal_ctx_set_dbus_connection (ctx, dbus_conn);

        if (!libhal_ctx_init (ctx, &error))
        {
          g_warning ("Could not initalize he HAL context: %s\n",
                     error.message);

          if (dbus_error_is_set (&error))
            dbus_error_free (&error);

          libhal_ctx_free (ctx);
          ctx = NULL;
        }
      }
    }
  }
#else /* USE_HAL5 */
  LibHalFunctions hal_functions = {
    NULL,                       /* mainloop integration */
    NULL,                       /* device_added */
    NULL,                       /* device_removed */
    NULL,                       /* device_new_capability */
    NULL,                       /* device_lost_capability */
    NULL,                       /* property_modified */
    NULL,                       /* device_condition */
  };

  if (ctx == NULL)
    ctx = hal_initialize (&hal_functions, FALSE);
#endif /* USE_HAL5 */

  return ctx;
}

GList *
hal_scan (gboolean recorder_only)
{
  GList *drives = NULL;
  int i;
  int num_devices;
  char **device_names;
  LibHalContext *ctx;

  ctx = get_hal_context ();
  if (ctx == NULL)
    return NULL;

#ifdef USE_HAL5
  device_names = libhal_find_device_by_capability (ctx, "storage.cdrom", &num_devices, NULL);
#else
  device_names = hal_find_device_by_capability (ctx, "storage.cdrom", &num_devices);
#endif

  if (device_names == NULL)
    return NULL;

  for (i = 0; i < num_devices; i++)
  {
    NautilusBurnDrive *drive;
    char *string;
    gboolean is_cdr;

    /*
     * Is it a CD burner? 
     */
    is_cdr = GET_BOOL_PROP ("storage.cdrom.cdr");

    drive = nautilus_burn_drive_new ();
    drive->type = NAUTILUS_BURN_DRIVE_TYPE_CD_DRIVE;
    if (is_cdr != FALSE)
      drive->type |= NAUTILUS_BURN_DRIVE_TYPE_CD_RECORDER;
    if (GET_BOOL_PROP ("storage.cdrom.cdrw"))
      drive->type |= NAUTILUS_BURN_DRIVE_TYPE_CDRW_RECORDER;
    if (GET_BOOL_PROP ("storage.cdrom.dvd"))
    {
      drive->type |= NAUTILUS_BURN_DRIVE_TYPE_DVD_DRIVE;

      if (GET_BOOL_PROP ("storage.cdrom.dvdram"))
        drive->type |= NAUTILUS_BURN_DRIVE_TYPE_DVD_RAM_RECORDER;
      if (GET_BOOL_PROP ("storage.cdrom.dvdr"))
        drive->type |= NAUTILUS_BURN_DRIVE_TYPE_DVD_RW_RECORDER;
      if (GET_BOOL_PROP ("storage.cdrom.dvd"))
        drive->type |= NAUTILUS_BURN_DRIVE_TYPE_DVD_DRIVE;
      if (GET_BOOL_PROP ("storage.cdrom.dvdplusr"))
        drive->type |= NAUTILUS_BURN_DRIVE_TYPE_DVD_PLUS_R_RECORDER;
      if (GET_BOOL_PROP ("storage.cdrom.dvdplusrw"))
        drive->type |= NAUTILUS_BURN_DRIVE_TYPE_DVD_PLUS_RW_RECORDER;
    }

#ifdef USE_HAL5
    drive->device = libhal_device_get_property_string (ctx, device_names[i], "block.device", NULL);
#else
    drive->device = hal_device_get_property_string (ctx, device_names[i], "block.device");
#endif
    drive->cdrecord_id = g_strdup (drive->device);

#ifdef USE_HAL5
    string = libhal_device_get_property_string (ctx, device_names[i], "storage.model", NULL);
#else
    string = hal_device_get_property_string (ctx, device_names[i], "storage.model");
#endif
    if (string != NULL)
      drive->display_name = string;
    else
      drive->display_name = g_strdup_printf ("Unnamed Drive (%s)", drive->device);

#ifdef USE_HAL5
    drive->max_speed_read = libhal_device_get_property_int (ctx, device_names[i], "storage.cdrom.read_speed", NULL) / CD_ROM_SPEED;
#else
    drive->max_speed_read = hal_device_get_property_int (ctx, device_names[i], "storage.cdrom.read_speed") / CD_ROM_SPEED;
#endif

#ifdef USE_HAL5
    if (libhal_device_property_exists (ctx, device_names[i], "storage.cdrom.write_speed", NULL))
      drive->max_speed_write = libhal_device_get_property_int (ctx, device_names[i], "storage.cdrom.write_speed", NULL) / CD_ROM_SPEED;
#else
    if (hal_device_property_exists (ctx, device_names[i], "storage.cdrom.write_speed"))
      drive->max_speed_write = hal_device_get_property_int (ctx, device_names[i], "storage.cdrom.write_speed") / CD_ROM_SPEED;
#endif

    nautilus_burn_drive_add_whitelist (drive);

    if (recorder_only && !(drive->type & NAUTILUS_BURN_DRIVE_TYPE_CD_RECORDER))
      nautilus_burn_drive_free (drive);
    else
      drives = g_list_prepend (drives, drive);

    drive->udi = g_strdup (device_names[i]);
  }

#ifdef USE_HAL5
  libhal_free_string_array (device_names);
#else
  hal_free_string_array (device_names);
#endif

  drives = g_list_reverse (drives);

  return drives;
}

#endif /* USE_HAL */

