/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 8; tab-width: 8 -*-
 *
 * nautilus-burn-drive.c: easy to use cd burner software
 *
 * Copyright (C) 2002-2004 Red Hat, Inc.
 * Copyright (C) 2005 William Jon McCann <mccann@jhu.edu>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 * 
 * Authors: Alexander Larsson <alexl@redhat.com>
 *          Bastien Nocera <hadess@hadess.net>
 *          William Jon McCann <mccann@jhu.edu>
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <errno.h>
#include <fcntl.h>
#include <stdio.h>
#include <unistd.h>
#include <sys/ioctl.h>

#ifdef HAVE_SYS_CDIO_H
#include <sys/cdio.h>
#endif

#include "nautilus-burn-drive.h"
#include "nautilus-burn-drive-common.h"

#ifdef __linux__
#include "nautilus-burn-drive-linux.h"
#endif /* __linux__ */

#ifdef __FreeBSD__
#include "nautilus-burn-drive-freebsd.h"
#endif /* __FreeBSD__ */

#ifdef USE_HAL
#include "nautilus-burn-drive-hal.h"
#endif /* USE_HAL */

/* For dvd_plus_rw_utils.cpp */
int get_mmc_profile      (int        fd);
int get_disc_status      (int        fd, 
                          int        *empty, 
                          int        *is_rewritable, 
                          int        *is_blank);

static gboolean
drive_door_is_open (int fd)
{
#ifdef __linux__
  {
    int status;

    status = ioctl (fd, CDROM_DRIVE_STATUS, CDSL_CURRENT);
    if (status < 0)
      return FALSE;

    return status == CDS_TRAY_OPEN;
  }
#else
  return FALSE;
#endif
}

static NautilusBurnMediaType
nautilus_burn_drive_get_media_type_from_path_full (const char *device, 
    gboolean * is_rewritable, gboolean * is_blank, gboolean * has_data, gboolean * has_audio)
{
  gpointer ioctl_handle;
  int mmc_profile;
  int fd;

  g_return_val_if_fail (device != NULL, NAUTILUS_BURN_MEDIA_TYPE_ERROR);

  if (is_rewritable)
    *is_rewritable = FALSE;
  if (is_blank)
    *is_blank = FALSE;
  if (has_data)
    *has_data = FALSE;
  if (has_audio)
    *has_audio = FALSE;

  ioctl_handle = open_ioctl_handle (device);
  if (ioctl_handle == INVALID_HANDLE)
  {
    if (errno == EBUSY)
      return NAUTILUS_BURN_MEDIA_TYPE_BUSY;
    return NAUTILUS_BURN_MEDIA_TYPE_ERROR;
  }

  fd = get_ioctl_handle_fd (ioctl_handle);
  mmc_profile = get_mmc_profile (fd);

  /*
   * Couldn't get the data about the media 
   */
  if (mmc_profile < 0)
  {
    gboolean opened;

    opened = drive_door_is_open (fd);

    if (opened != FALSE)
    {
      close_ioctl_handle (ioctl_handle);
      return NAUTILUS_BURN_MEDIA_TYPE_ERROR;
    }
    else
    {
      int blank, rewrite, empty;

      if (get_disc_status (fd, &empty, &rewrite, &blank) == 0)
      {
        close_ioctl_handle (ioctl_handle);

        if (is_rewritable)
          *is_rewritable = rewrite;
        if (is_blank)
          *is_blank = blank;
        if (has_data)
          *has_data = !blank;
        if (has_audio)
          *has_audio = FALSE;

        if (empty)
          return NAUTILUS_BURN_MEDIA_TYPE_ERROR;
        else
          return NAUTILUS_BURN_MEDIA_TYPE_UNKNOWN;
      }

      close_ioctl_handle (ioctl_handle);

      return NAUTILUS_BURN_MEDIA_TYPE_UNKNOWN;
    }
  }

  close_ioctl_handle (ioctl_handle);

  if (is_blank)
    *is_blank = mmc_profile & 0x10000;
   if (has_data)
     *has_data = !(mmc_profile & 0x10000);
   if (has_audio)
     *has_audio = FALSE;

  switch (mmc_profile & 0xFFFF)
  {
    case -1:
      g_assert_not_reached ();
    case 0:                    /* No Media */
      return NAUTILUS_BURN_MEDIA_TYPE_ERROR;
    case 0x8:                  /* Commercial CDs and Audio CD  */
      return NAUTILUS_BURN_MEDIA_TYPE_CD;
    case 0x9:                  /* CD-R                         */
      return NAUTILUS_BURN_MEDIA_TYPE_CDR;
    case 0xa:                  /* CD-RW      */
      if (is_rewritable)
        *is_rewritable = TRUE;
      return NAUTILUS_BURN_MEDIA_TYPE_CDRW;
    case 0x10:                 /* Commercial DVDs    */
      return NAUTILUS_BURN_MEDIA_TYPE_DVD;
    case 0x11:                 /* DVD-R                        */
      return NAUTILUS_BURN_MEDIA_TYPE_DVDR;
    case 0x13:                 /* DVD-RW Restricted Overwrite  */
    case 0x14:                 /* DVD-RW Sequential            */
      if (is_rewritable)
        *is_rewritable = TRUE;
      return NAUTILUS_BURN_MEDIA_TYPE_DVDRW;
    case 0x1B:                 /* DVD+R                        */
      return NAUTILUS_BURN_MEDIA_TYPE_DVD_PLUS_R;
    case 0x2B:                 /* DVD+R Double Layer           */
      return NAUTILUS_BURN_MEDIA_TYPE_DVD_PLUS_R_DL;
    case 0x1A:                 /* DVD+RW                       */
      if (is_rewritable)
        *is_rewritable = TRUE;
      return NAUTILUS_BURN_MEDIA_TYPE_DVD_PLUS_RW;
    case 0x12:                 /* DVD-RAM                      */
      return NAUTILUS_BURN_MEDIA_TYPE_DVD_RAM;
    default:
      return NAUTILUS_BURN_MEDIA_TYPE_UNKNOWN;
  }
}

/**
 * nautilus_burn_drive_new:
 *
 * Create a new #NautilusBurnDrive.
 *
 * Return value: The new drive.
 *
 * Since: 2.8
 **/
NautilusBurnDrive *
nautilus_burn_drive_new (void)
{
  NautilusBurnDrive *drive;

  drive = g_new0 (NautilusBurnDrive, 1);

  return drive;
}

/**
 * nautilus_burn_drive_free:
 * @drive: #NautilusBurnDrive to be freed
 *
 * Free @drive.
 **/
void
nautilus_burn_drive_free (NautilusBurnDrive * drive)
{
  g_return_if_fail (drive != NULL);

  g_free (drive->display_name);
  g_free (drive->cdrecord_id);
  g_free (drive->device);
  g_free (drive->udi);
  g_free (drive);
}

/**
 * nautilus_burn_drive_get_media_type_from_path:
 * @device: Filename of drive device (eg. "/dev/hdb")
 *
 * Determine the type of the media in the drive at device @device.
 *
 * Return value: The type of the media in the drive.
 **/
NautilusBurnMediaType
nautilus_burn_drive_get_media_type_from_path (const char *device)
{
  g_return_val_if_fail (device != NULL, NAUTILUS_BURN_MEDIA_TYPE_ERROR);

  return nautilus_burn_drive_get_media_type_from_path_full (device, NULL, NULL, NULL, NULL);
}

gboolean
nautilus_burn_drive_door_is_open (NautilusBurnDrive *drive)
{
  gpointer ioctl_handle;
  int      fd;
  gboolean ret;

  g_return_val_if_fail (drive != NULL, FALSE);

  ioctl_handle = open_ioctl_handle (drive->device);
  if (ioctl_handle == INVALID_HANDLE)
    return FALSE;

  fd = get_ioctl_handle_fd (ioctl_handle);

  ret = drive_door_is_open (fd);

  close_ioctl_handle (ioctl_handle);

  return ret;
}

/**
 * nautilus_burn_drive_get_list:
 * @recorder_only: Include only devices capable of recording
 * @add_image: Include a file image device
 *
 * Find media devices on the system.
 *
 * Return value: List of media drives available.
 **/
GList *
nautilus_burn_drive_get_list (void) /*(gboolean recorder_only, gboolean add_image) */
{
  GList *drives = NULL;

#ifdef USE_HAL
  drives = hal_scan (FALSE) /*(recorder_only) */ ;
#endif

  if (drives == NULL)
  {
#if defined (__linux__)
    drives = linux_scan (FALSE) /*(recorder_only) */ ;
#elif defined (__FreeBSD__)
    drives = freebsd_scan (FALSE) /*(recorder_only) */ ;
#else
    drives = cdrecord_scan (FALSE) /*(recorder_only) */ ;
#endif
  }
/*
	if (add_image) {
		NautilusBurnDrive *drive;
		drive = nautilus_burn_drive_get_file_image ();
		drives = g_list_append (drives, drive);
	}
*/
  return drives;
}

gboolean
nautilus_burn_drive_eject (NautilusBurnDrive *drive)
{
  gpointer ioctl_handle;
  int      fd;
  gboolean ret;

  g_return_val_if_fail (drive != NULL, FALSE);

  ioctl_handle = open_ioctl_handle (drive->device);
  if (ioctl_handle == INVALID_HANDLE)
    return FALSE;

  fd = get_ioctl_handle_fd (ioctl_handle);

  ret = ioctl (fd, CDROMEJECT);

  close_ioctl_handle (ioctl_handle);

  return ret == 0;
}

