/* OGMRip - A library for DVD ripping and encoding
 * Copyright (C) 2004-2005 Olivier Rolland <billl@esil.univ-mrs.fr>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#include "ogmrip-container.h"
#include "ogmrip-backend.h"
#include "ogmsp-enums.h"
#include "ogmsp-exec.h"

#include <unistd.h>
#include <glib/gstdio.h>

#define OGMRIP_CONTAINER_GET_PRIVATE(o) \
  (G_TYPE_INSTANCE_GET_PRIVATE ((o), OGMRIP_TYPE_CONTAINER, OGMRipContainerPriv))

struct _OGMRipContainerPriv
{
  gchar *title;
  gchar *output;

  guint tsize;
  guint tnumber;

  gchar *video;
  GList *audio;
  GList *chapters;
  GList *subp;

  gboolean do_unlink;
};

static void ogmrip_container_finalize (GObject *gobject);

G_DEFINE_ABSTRACT_TYPE (OGMRipContainer, ogmrip_container, OGMSP_TYPE_BIN)

static void
ogmrip_container_class_init (OGMRipContainerClass *klass)
{
  GObjectClass *gobject_class;
  OGMSpSpawnClass *spawn_class;

  gobject_class = G_OBJECT_CLASS (klass);
  spawn_class = OGMSP_SPAWN_CLASS (klass);

  gobject_class->finalize = ogmrip_container_finalize;

  g_type_class_add_private (klass, sizeof (OGMRipContainerPriv));
}

static void
ogmrip_container_init (OGMRipContainer *container)
{
  container->priv = OGMRIP_CONTAINER_GET_PRIVATE (container);
  container->priv->chapters = NULL;
  container->priv->tnumber = 1;
  container->priv->tsize = 0;
}

static void
ogmrip_stream_free (gchar **strv, gboolean do_unlink)
{
  if (do_unlink)
    g_unlink (strv[0]);

  g_free (strv[0]);
  g_free (strv);
}

static void
ogmrip_container_finalize (GObject *gobject)
{
  OGMRipContainer *container;

  container = OGMRIP_CONTAINER (gobject);

  if (container->priv->do_unlink && container->priv->video)
    g_unlink (container->priv->video);

  g_list_foreach (container->priv->audio, (GFunc) ogmrip_stream_free, 
      GINT_TO_POINTER (container->priv->do_unlink));
  g_list_free (container->priv->audio);

  g_list_foreach (container->priv->chapters, (GFunc) ogmrip_stream_free, 
      GINT_TO_POINTER (container->priv->do_unlink));
  g_list_free (container->priv->chapters);

  g_list_foreach (container->priv->subp, (GFunc) ogmrip_stream_free, 
      GINT_TO_POINTER (container->priv->do_unlink));
  g_list_free (container->priv->subp);

  g_free (container->priv->title);
  g_free (container->priv->output);
  g_free (container->priv->video);

  G_OBJECT_CLASS (ogmrip_container_parent_class)->finalize (gobject);
}

G_CONST_RETURN gchar *
ogmrip_container_get_output (OGMRipContainer *container)
{
  g_return_val_if_fail (OGMRIP_IS_CONTAINER (container), NULL);

  return container->priv->output;
}

void
ogmrip_container_set_output (OGMRipContainer *container, const gchar *output)
{
  g_return_if_fail (OGMRIP_IS_CONTAINER (container));
  g_return_if_fail (output && *output);

  g_free (container->priv->output);
  container->priv->output = g_strdup (output);
}

void
ogmrip_container_set_title (OGMRipContainer *container, const gchar *title)
{
  g_return_if_fail (OGMRIP_IS_CONTAINER (container));

  g_free (container->priv->title);
  if (title)
    container->priv->title = g_strdup (title);
  else
    container->priv->title = NULL;
}

G_CONST_RETURN gchar *
ogmrip_container_get_title (OGMRipContainer *container)
{
  g_return_val_if_fail (OGMRIP_IS_CONTAINER (container), NULL);

  return container->priv->title;
}

G_CONST_RETURN gchar *
ogmrip_container_get_video (OGMRipContainer *container)
{
  g_return_val_if_fail (OGMRIP_IS_CONTAINER (container), NULL);

  return container->priv->video;
}

void
ogmrip_container_set_video (OGMRipContainer *container, const gchar *input)
{
  g_return_if_fail (OGMRIP_IS_CONTAINER (container));
  g_return_if_fail (input && *input);

  g_free (container->priv->video);
  container->priv->video = g_strdup (input);
}

void
ogmrip_container_add_audio (OGMRipContainer *container, const gchar *input, gint language)
{
  gchar **strv;

  g_return_if_fail (OGMRIP_IS_CONTAINER (container));
  g_return_if_fail (input && *input);

  strv = g_new0 (gchar *, 2); 

  strv[0] = g_strdup (input);
  strv[1] = GINT_TO_POINTER (language);

  container->priv->audio = g_list_append (container->priv->audio, strv);
}

void
ogmrip_container_add_chapters (OGMRipContainer *container, const gchar *input, gint language)
{
  gchar **strv;

  g_return_if_fail (OGMRIP_IS_CONTAINER (container));
  g_return_if_fail (input && *input);

  strv = g_new0 (gchar *, 2); 

  strv[0] = g_strdup (input);
  strv[1] = GINT_TO_POINTER (language);

  container->priv->chapters = g_list_append (container->priv->chapters, strv);
}

void
ogmrip_container_add_subp (OGMRipContainer *container, const gchar *input, gint language)
{
  gchar **strv;

  g_return_if_fail (OGMRIP_IS_CONTAINER (container));
  g_return_if_fail (input && *input);

  strv = g_new0 (gchar *, 2); 

  strv[0] = g_strdup (input);
  strv[1] = GINT_TO_POINTER (language);

  container->priv->subp = g_list_append (container->priv->subp, strv);
}

void
ogmrip_container_foreach_audio (OGMRipContainer *container, OGMRipContainerFunc func, gpointer data)
{
  GList *link;
  gchar **strv;
  struct stat buf; 

  g_return_if_fail (OGMRIP_IS_CONTAINER (container));
  g_return_if_fail (func != NULL);

  for (link = container->priv->audio; link; link = link->next)
  {
    strv = (gchar **) link->data;
    if (g_file_test (strv[0], G_FILE_TEST_IS_REGULAR))
      if (g_stat (strv[0], &buf) == 0 && buf.st_size > 0)
        func (container, strv[0], GPOINTER_TO_INT (strv[1]), data);
  }
}

void
ogmrip_container_foreach_chapter (OGMRipContainer *container, OGMRipContainerFunc func, gpointer data)
{
  GList *link;
  gchar **strv;
  struct stat buf; 

  g_return_if_fail (OGMRIP_IS_CONTAINER (container));
  g_return_if_fail (func != NULL);

  for (link = container->priv->chapters; link; link = link->next)
  {
    strv = (gchar **) link->data;
    if (g_file_test (strv[0], G_FILE_TEST_IS_REGULAR))
      if (g_stat (strv[0], &buf) == 0 && buf.st_size > 0)
        func (container, strv[0], GPOINTER_TO_INT (strv[1]), data);
  }
}

void
ogmrip_container_foreach_subp (OGMRipContainer *container, OGMRipContainerFunc func, gpointer data)
{
  GList *link;
  gchar **strv;
  struct stat buf; 

  g_return_if_fail (OGMRIP_IS_CONTAINER (container));
  g_return_if_fail (func != NULL);

  for (link = container->priv->subp; link; link = link->next)
  {
    strv = (gchar **) link->data;
    if (g_file_test (strv[0], G_FILE_TEST_IS_REGULAR))
      if (g_stat (strv[0], &buf) == 0 && buf.st_size > 0)
        func (container, strv[0], GPOINTER_TO_INT (strv[1]), data);
  }
}

void
ogmrip_container_set_target_size (OGMRipContainer *container, guint tnumber, guint tsize)
{
  g_return_if_fail (OGMRIP_IS_CONTAINER (container));
  g_return_if_fail (tnumber == 1 || tsize != 0);

  container->priv->tsize = tsize;
  container->priv->tnumber = tnumber;
}

gint
ogmrip_container_get_tsize (OGMRipContainer *container)
{
  g_return_val_if_fail (OGMRIP_IS_CONTAINER (container), -1);

  return container->priv->tsize;
}

gint
ogmrip_container_get_tnumber (OGMRipContainer *container)
{
  g_return_val_if_fail (OGMRIP_IS_CONTAINER (container), -1);

  return container->priv->tnumber;
}

static gint64
ogmrip_container_get_file_size (const gchar *filename)
{
  struct stat buf;
  gint64 size = 0;

  if (g_file_test (filename, G_FILE_TEST_IS_REGULAR))
    if (g_stat (filename, &buf) == 0)
      size = (gint64) buf.st_size;

  return size;
}

gint64
ogmrip_container_get_nonvideo_size (OGMRipContainer *container)
{
  GList *link;
  gchar **strv;
  gint64 nvsize = 0;

  g_return_val_if_fail (OGMRIP_IS_CONTAINER (container), -1);

  for (link = container->priv->chapters; link; link = link->next)
  {
    strv = (gchar **) link->data;
    nvsize += ogmrip_container_get_file_size (strv[0]);
  }
  
  for (link = container->priv->audio; link; link = link->next)
  {
    strv = (gchar **) link->data;
    nvsize += ogmrip_container_get_file_size (strv[0]);
  }
  
  for (link = container->priv->subp; link; link = link->next)
  {
    strv = (gchar **) link->data;
    nvsize += ogmrip_container_get_file_size (strv[0]);
  }
  
  return nvsize;
}

gint64
ogmrip_container_get_overhead_size (OGMRipContainer *container, guint64 size, gulong length)
{
  OGMRipContainerClass *klass;

  g_return_val_if_fail (OGMRIP_IS_CONTAINER (container), -1);
  g_return_val_if_fail (size != 0 && length != 0, -1);

  klass = OGMRIP_CONTAINER_GET_CLASS (container);

  if (klass->get_overhead_size)
    return (* klass->get_overhead_size) (container, size, length);

  return 0;
}

GList *
ogmrip_container_get_audio (OGMRipContainer *container)
{
  g_return_val_if_fail (OGMRIP_IS_CONTAINER (container), NULL);

  return g_list_copy (container->priv->audio);
}

gint
ogmrip_container_count_audio (OGMRipContainer *container)
{
  g_return_val_if_fail (OGMRIP_IS_CONTAINER (container), -1);

  return g_list_length (container->priv->audio);
}

GList *
ogmrip_container_get_subp (OGMRipContainer *container)
{
  g_return_val_if_fail (OGMRIP_IS_CONTAINER (container), NULL);

  return g_list_copy (container->priv->subp);
}

gint
ogmrip_container_count_subp (OGMRipContainer *container)
{
  g_return_val_if_fail (OGMRIP_IS_CONTAINER (container), -1);

  return g_list_length (container->priv->subp);
}

GList *
ogmrip_container_get_chapters (OGMRipContainer *container)
{
  g_return_val_if_fail (OGMRIP_IS_CONTAINER (container), NULL);

  return g_list_copy (container->priv->chapters);
}

gint
ogmrip_container_count_chapters (OGMRipContainer *container)
{
  g_return_val_if_fail (OGMRIP_IS_CONTAINER (container), -1);

  return g_list_length (container->priv->chapters);
}

void
ogmrip_container_set_unlink_on_unref (OGMRipContainer *container, gboolean do_unlink)
{
  g_return_if_fail (OGMRIP_IS_CONTAINER (container));

  container->priv->do_unlink = do_unlink;
}

