/* OGMRip - A DVD Encoder for GNOME
 * Copyright (C) 2004-2005 Olivier Rolland <billl@users.sf.fr>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#include "config.h"

#include "ogmrip-helper.h"

#include <glib/gi18n.h>
#include <glib/gstdio.h>
#include <sys/ioctl.h>
#include <unistd.h>
#include <string.h>
#include <stdlib.h>
#include <fcntl.h>
#include <errno.h>

#ifdef HAVE_SYS_STATVFS_H
#include <sys/statvfs.h>
#endif

#if defined(__FreeBSD__) || defined(__NetBSD__)
#include <sys/cdio.h>
#define CDROMEJECT CDIOCEJECT
#if __FreeBSD_version < 500000
#undef MIN
#undef MAX
#include <sys/param.h>
#include <sys/mount.h>
#endif /* __FreeBSD_version */
#endif /* __FreeBSD__ */

#ifdef __linux__
#include <linux/cdrom.h>
#endif /* __linux__ */

static gchar *tmp_dir = NULL;

gboolean
g_file_rename (const gchar *old_name, const gchar *new_name, GError **error)
{
  g_return_val_if_fail (old_name != NULL, FALSE);
  g_return_val_if_fail (new_name != NULL, FALSE);
  g_return_val_if_fail (error == NULL || *error == NULL, FALSE);

  if (strcmp (old_name, new_name) == 0)
    return TRUE;

  if (g_file_test (new_name, G_FILE_TEST_EXISTS))
  {
    if (!g_file_test (new_name, G_FILE_TEST_IS_REGULAR))
      return FALSE;
    if (g_unlink (new_name) < 0)
      return FALSE;
  }

  if (g_rename (old_name, new_name) < 0)
    return FALSE;

  return TRUE;
}

gchar *
g_fs_get_mount_point (const gchar *filename)
{
  gchar *cwd, *dirname, *mp = NULL;
  struct stat cur_stat, last_stat;

  g_return_val_if_fail (filename && *filename, NULL);

  cwd = g_get_current_dir ();

  if (g_file_test (filename, G_FILE_TEST_IS_DIR))
    dirname = g_strdup (filename);
  else
    dirname = g_path_get_dirname (filename);

  if (!g_file_test (dirname, G_FILE_TEST_EXISTS))
    goto done;

  if (g_stat (dirname, &last_stat) < 0)
    goto done;

  if (chdir (dirname) < 0)
    goto done;

  for (;;)
  {
    if (g_stat ("..", &cur_stat) < 0)
      goto done;

    if (cur_stat.st_dev != last_stat.st_dev || cur_stat.st_ino == last_stat.st_ino)
      break;

    if (chdir ("..") < 0)
      goto done;

    last_stat = cur_stat;
  }

  mp = g_get_current_dir ();

done:
  chdir (cwd);
  g_free (cwd);
  g_free (dirname);

  return mp;
}

gint64
g_fs_space_left (const gchar *filename)
{
  gint status;
  gchar *dirname;
#ifdef HAVE_STATVFS
  struct statvfs statfs_buf;
#else
  struct statfs statfs_buf;
#endif

  g_return_val_if_fail (filename && *filename, -1);

  if (g_file_test (filename, G_FILE_TEST_IS_DIR))
    dirname = g_strdup (filename);
  else
    dirname = g_path_get_dirname (filename);

  if (!g_file_test (dirname, G_FILE_TEST_EXISTS))
  {
    g_free (dirname);
    return -1;
  }

#ifdef HAVE_STATVFS
  status = statvfs (dirname, &statfs_buf);
#else
  status = statfs (dirname, &statfs_buf);
#endif

  g_free (dirname);

  if (status < 0)
    return -1;

#ifdef OGMRIP_DEBUG
  g_message ("Space left on device containing '%s': %" G_GINT64_FORMAT " bytes", 
      filename, (gint64) statfs_buf.f_bsize * (gint64) statfs_buf.f_bavail);
#endif

  return (gint64) statfs_buf.f_bsize * (gint64) statfs_buf.f_bavail;
}

gboolean
g_dir_remove (const gchar *path, gboolean recursive, GError **error)
{
  g_return_val_if_fail (path && *path, FALSE);
  g_return_val_if_fail (error == NULL || *error == NULL, FALSE);
  g_return_val_if_fail (g_file_test (path, G_FILE_TEST_IS_DIR), FALSE);

  if (recursive)
  {
    GDir *dir;
    GError *tmp_error = NULL;

    gchar *filename;
    const gchar *name;

    dir = g_dir_open (path, 0, &tmp_error);
    if (!dir)
    {
      g_propagate_error (error, tmp_error);
      return FALSE;
    }

    while ((name = g_dir_read_name (dir)))
    {
      filename = g_build_filename (path, name, NULL);
      if (g_file_test (filename, G_FILE_TEST_IS_DIR))
      {
        if (!g_dir_remove (filename, TRUE, &tmp_error))
        {
          if (tmp_error)
            g_propagate_error (error, tmp_error);
          g_free (filename);
          return FALSE;
        }
      }
      else
      {
        if (g_unlink (filename) != 0)
        {
          g_free (filename);
          return FALSE;
        }
      }
      g_free (filename);
    }

    g_dir_close (dir);
  }

  if (g_rmdir (path) != 0)
    return FALSE;

  return TRUE;
}

gboolean
g_dir_make (const gchar *path, mode_t mode, GError **error)
{
  const gchar *sep;
  gchar *current;

  g_return_val_if_fail (path && *path, FALSE);
  g_return_val_if_fail (error == NULL || *error == NULL, FALSE);

  for (sep = path; *sep; sep ++)
  {
    for (; *sep; sep ++)
      if (*sep == G_DIR_SEPARATOR)
        break;

    if (sep - path > 0)
    {
      current = g_strndup (path, sep - path);

      if (g_mkdir (current, mode) < 0)
      {
        g_set_error (error, OGMRIP_ERROR, OGMRIP_ERROR_INTERNAL, 
            _("Could not create directory %s: %s"), path, g_strerror (errno));
        g_free (current);
        return FALSE;
      }
      g_free (current);
    }
  }
  return TRUE;
}

gchar *
g_dir_make_temp (const gchar *tmpl, mode_t mode, GError **error)
{
  gchar *path;

  g_return_val_if_fail (tmpl && *tmpl, FALSE);
  g_return_val_if_fail (error == NULL || *error == NULL, FALSE);

  path = g_build_filename (tmp_dir, tmpl, NULL);

  if (!mkdtemp (path))
  {
    g_free (path);
    return NULL;
  }

  return path;
}

void
g_file_unref (gchar *filename, gboolean do_unlink)
{
  if (filename)
  {
    if (do_unlink)
      g_unlink (filename);
    g_free (filename);
  }
}

gchar *
g_file_make_temp (const gchar *tmpl, GError **error)
{
  gchar *filename;
  int fd;

  g_return_val_if_fail (error == NULL || *error == NULL, NULL);

  filename = g_build_filename (tmp_dir, tmpl, NULL);
  fd = g_mkstemp (filename);
  if (fd < 0)
  {
    g_free (filename);
    return NULL;
  }

  return filename;
}

gint
gconf_client_get_int_default (GConfClient *client, const gchar *key, gint def)
{
  GConfValue* val;

  val = gconf_client_get (client, key, NULL);
  if (!val)
    gconf_client_set_int (client, key, def, NULL);
  else
  {
    def = gconf_value_get_int (val);
    gconf_value_free (val);
  }

  return def;
}

gboolean
gconf_client_get_bool_default (GConfClient *client, const gchar *key, gboolean def)
{
  GConfValue* val;

  val = gconf_client_get (client, key, NULL);
  if (!val)
    gconf_client_set_bool (client, key, def, NULL);
  else
  {
    def = gconf_value_get_bool (val);
    gconf_value_free (val);
  }

  return def;
}

gchar *
gconf_client_get_string_default (GConfClient *client, const gchar *key, const gchar *def)
{
  GConfValue* val;
  gchar *str;

  val = gconf_client_get (client, key, NULL);
  if (!val)
  {
    gconf_client_set_string (client, key, def, NULL);
    str = g_strdup (def);
  }
  else
  {
    str = g_strdup (gconf_value_get_string (val));
    gconf_value_free (val);
  }

  return str;
}

void
gtk_window_set_parent (GtkWindow *window, GtkWindow *parent)
{
  g_return_if_fail (GTK_IS_WINDOW (window));
  g_return_if_fail (GTK_IS_WINDOW (parent));
  g_return_if_fail (window != parent);

  gtk_window_set_transient_for (window, parent);
  gtk_window_set_position (window, GTK_WIN_POS_CENTER_ON_PARENT);
  gtk_window_set_gravity (window, GDK_GRAVITY_CENTER);
  gtk_window_set_destroy_with_parent (window, TRUE);
}

void
gtk_window_set_icon_from_stock (GtkWindow *window, const gchar *stock_id)
{
  GdkPixbuf *pixbuf;

  g_return_if_fail (GTK_IS_WINDOW (window));
  g_return_if_fail (stock_id && *stock_id);
  
  pixbuf = gtk_widget_render_icon (GTK_WIDGET (window), stock_id, GTK_ICON_SIZE_DIALOG, NULL);
  gtk_window_set_icon (window, pixbuf);
  gdk_pixbuf_unref (pixbuf);
}

gint
gtk_radio_button_get_active (GtkRadioButton *radio)
{
  GSList *link;
  gint i;

  g_return_val_if_fail (GTK_IS_RADIO_BUTTON (radio), -1);

  link = gtk_radio_button_get_group (GTK_RADIO_BUTTON (radio));
  for (i = 0;  link; i++, link = link->next)
    if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (link->data)))
      return i;

  return -1;
}

void
gtk_radio_button_set_active (GtkRadioButton *radio, guint index)
{
  GSList *link;
  gint i;

  g_return_if_fail (GTK_IS_RADIO_BUTTON (radio));

  link = gtk_radio_button_get_group (GTK_RADIO_BUTTON (radio));
  for (i = 0;  link; i++, link = link->next)
    if (i == index)
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (link->data), TRUE);
}

gboolean
gtk_tree_model_iter_prev (GtkTreeModel *tree_model, GtkTreeIter *iter)
{
  gboolean retval = FALSE;
  GtkTreePath *path;

  path = gtk_tree_model_get_path (tree_model, iter);
  if (path)
  {
    if (gtk_tree_path_prev (path))
      retval = gtk_tree_model_get_iter (tree_model, iter, path);
    gtk_tree_path_free (path);
  }

  return retval;
}

void
gtk_label_set_int (GtkLabel *label, gint value)
{
  gchar *text;

  g_return_if_fail (GTK_IS_LABEL (label));

  text = g_strdup_printf ("%d", value);
  gtk_label_set_text (label, text);
  g_free (text);
}

gint
gtk_label_get_int (GtkLabel *label)
{
  const gchar *text;

  g_return_val_if_fail (GTK_IS_LABEL (label), G_MININT);

  text = gtk_label_get_text (label);
  
  return atoi (text);
}

GtkWidget *
gtk_box_get_child (GtkBox *box, guint n)
{
  GtkBoxChild *child;

  g_return_val_if_fail (GTK_IS_BOX (box), NULL);

  child = g_list_nth_data (box->children, n);
  if (!child)
    return NULL;

  return child->widget;
}

void
ogmrip_statusbar_push (GtkStatusbar *statusbar, const gchar *text)
{
  guint id;

  g_return_if_fail (GTK_IS_STATUSBAR (statusbar));
  g_return_if_fail (text != NULL);

  id = gtk_statusbar_get_context_id (statusbar, "__menu_hint__");
  gtk_statusbar_push (statusbar, id, text);
}

void
ogmrip_statusbar_pop (GtkStatusbar *statusbar)
{
  guint id;

  g_return_if_fail (GTK_IS_STATUSBAR (statusbar));

  id = gtk_statusbar_get_context_id (statusbar, "__menu_hint__");
  gtk_statusbar_pop (statusbar, id);
}

gboolean
ogmdvd_tray_is_open (const gchar *device)
{
  gint fd;
#if defined(__linux__) || defined(__FreeBSD__)
  gint status = 0;
#endif
#ifdef __FreeBSD__
  struct ioc_toc_header h;
#endif

  g_return_val_if_fail (device && *device, FALSE);

  fd = g_open (device, O_RDONLY | O_NONBLOCK | O_EXCL, 0);

  if (fd < 0)
    return FALSE;

#if defined(__linux__)
  status = ioctl (fd, CDROM_DRIVE_STATUS, CDSL_CURRENT);
#elif defined(__FreeBSD__)
  status = ioctl (fd, CDIOREADTOCHEADER, &h);
#endif

  close (fd);

#if defined(__linux__)
  return status < 0 ? FALSE : status == CDS_TRAY_OPEN;
#elif defined(__FreeBSD__)
  return status < 0;
#else
  return FALSE;
#endif
}

gboolean
ogmdvd_tray_open (const gchar *device)
{
#if defined(__linux__) || defined(__FreeBSD__) || defined(__NetBSD__)
  gint fd, status;

  g_return_val_if_fail (device != NULL, FALSE);

  fd = g_open (device, O_RDONLY | O_NONBLOCK, 0);
  if (fd < 0)
    return FALSE;

#ifdef __FreeBSD__
  ioctl (fd, CDIOCALLOW);
#endif

  status = ioctl (fd, CDROMEJECT);
  if (status < 0)
  {
    close (fd);
    return FALSE;
  }

  close (fd);
#endif

  return TRUE;
}

gboolean
ogmdvd_is_valid_media (const gchar *device)
{
  NautilusBurnMediaType type;
#ifdef __linux__
  gint fd, status;
#endif

  g_return_val_if_fail (device && *device, FALSE);

  type = nautilus_burn_drive_get_media_type_from_path (device);

  if (type != NAUTILUS_BURN_MEDIA_TYPE_UNKNOWN && type < NAUTILUS_BURN_MEDIA_TYPE_DVD)
    return FALSE;

  if (type == NAUTILUS_BURN_MEDIA_TYPE_UNKNOWN)
    g_warning ("Unknown media type. Assuming it is valid.");

#ifdef __linux__

  fd = g_open (device, O_RDONLY | O_NONBLOCK | O_EXCL, 0);
  if (fd < 0)
    return FALSE;

  status = ioctl (fd, CDROM_DISC_STATUS, CDSL_CURRENT);
  close (fd);

  if (status < 0)
    return FALSE;

  return status != CDS_NO_INFO && status != CDS_NO_DISC;

#endif

  return TRUE;
}

GQuark
ogmrip_error_quark (void)
{
  static GQuark quark = 0;

  if (quark == 0)
    quark = g_quark_from_static_string ("ogmrip-error-quark");

  return quark;
}

gint
ogmrip_message_dialog (GtkWindow *parent, GtkMessageType type, const gchar *format, ...)
{
  GtkWidget *dialog = NULL;
  gchar *message;
  va_list args;

  g_return_val_if_fail (parent == NULL || GTK_IS_WINDOW (parent), GTK_RESPONSE_NONE);

  va_start (args, format);
  message = g_strdup_vprintf (format, args);
  va_end (args);

  switch (type)
  {
    case GTK_MESSAGE_ERROR:
      dialog = gtk_message_dialog_new (parent, 
          GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT, type, GTK_BUTTONS_CLOSE, message);
      gtk_window_set_icon_from_stock (GTK_WINDOW (dialog), GTK_STOCK_DIALOG_ERROR);
      break;
    case GTK_MESSAGE_QUESTION:
      dialog = gtk_message_dialog_new (parent, 
          GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT, type, GTK_BUTTONS_YES_NO, message);
      gtk_window_set_icon_from_stock (GTK_WINDOW (dialog), GTK_STOCK_DIALOG_QUESTION);
      break;
    case GTK_MESSAGE_INFO:
      dialog = gtk_message_dialog_new (parent, 
          GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT, type, GTK_BUTTONS_CLOSE, message);
      gtk_window_set_icon_from_stock (GTK_WINDOW (dialog), GTK_STOCK_DIALOG_INFO);
      break;
    case GTK_MESSAGE_WARNING:
      dialog = gtk_message_dialog_new (parent, 
          GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT, type, GTK_BUTTONS_CLOSE, message);
      gtk_window_set_icon_from_stock (GTK_WINDOW (dialog), GTK_STOCK_DIALOG_WARNING);
      break;
    default:
      break;
  }

  g_free (message);

  if (dialog)
  {
    gint response;

    gtk_window_set_gravity (GTK_WINDOW (dialog), GDK_GRAVITY_CENTER);
    gtk_window_set_position (GTK_WINDOW (dialog), GTK_WIN_POS_CENTER_ON_PARENT);
    gtk_label_set_use_markup (GTK_LABEL (GTK_MESSAGE_DIALOG (dialog)->label), TRUE);
    response = gtk_dialog_run (GTK_DIALOG (dialog));
    gtk_widget_destroy (dialog);

    return response;
  }

  return GTK_RESPONSE_NONE;
}

G_CONST_RETURN gchar *
ogmrip_get_tmp_dir (void)
{
  if (!tmp_dir)
    ogmrip_set_tmp_dir (g_get_tmp_dir ());

  return tmp_dir;
}

void
ogmrip_set_tmp_dir (const gchar *dir)
{
  g_return_if_fail (dir != NULL);
  g_return_if_fail (g_file_test (dir, G_FILE_TEST_IS_DIR));

  g_free (tmp_dir);
  tmp_dir = g_strdup (dir);
}

gint
ogmrip_file_open_tmp (const gchar *tmpl, gchar **name_used, GError **error)
{
  const gchar *tmpdir;
  gchar *fulltmpl;
  gint retval;

  g_return_val_if_fail (error == NULL || *error == NULL, -1);

  if (!tmpl)
    tmpl = ".XXXXXX";

  if (!g_str_has_suffix (tmpl, "XXXXXX"))
  {
    g_set_error (error, G_FILE_ERROR, G_FILE_ERROR_FAILED, 
        "Template '%s' doesn't end with XXXXXX", tmpl);
    return -1;
  }

  if ((strchr (tmpl, G_DIR_SEPARATOR)) != NULL 
#ifdef G_OS_WIN32
      || (strchr (tmpl, '/') != NULL)
#endif
      )
  {
    g_set_error (error, G_FILE_ERROR, G_FILE_ERROR_FAILED, 
        "Template '%s' invalid, should not contain a '/'", tmpl);
    return -1;
  }

  tmpdir = ogmrip_get_tmp_dir ();
  fulltmpl = g_build_filename (tmpdir, tmpl, NULL);

  retval = g_mkstemp (fulltmpl);
  if (retval < 0)
  {
    g_set_error (error, G_FILE_ERROR, g_file_error_from_errno (errno), 
        "Failed to create file '%s': %s", tmpl, g_strerror (errno));
    g_free (fulltmpl);
    return -1;
  }

  if (name_used)
    *name_used = fulltmpl;
  else
    g_free (fulltmpl);

  return retval;
}

void
ogmrip_app_init (GtkWindow *window, const gchar *domain)
{
  GtkUIManager   *ui_manager;
  GtkActionGroup *action_group;
  GtkAccelGroup  *accel_group;

  g_return_if_fail (GTK_IS_WINDOW (window));

  ui_manager = gtk_ui_manager_new ();
  g_object_set_data (G_OBJECT (window), "__ui_manager__", ui_manager);

  action_group = gtk_action_group_new ("MenuActions");
  g_object_set_data (G_OBJECT (window), "__action_group__", action_group);

  gtk_ui_manager_insert_action_group (ui_manager, action_group, 0);

  accel_group = gtk_ui_manager_get_accel_group (ui_manager);
  gtk_window_add_accel_group (GTK_WINDOW (window), accel_group);

  if (domain)
    gtk_action_group_set_translation_domain (action_group, domain);
}

guint
ogmrip_app_add_ui_from_file (GtkWindow *window, const gchar *filename, GError **error)
{
  GtkUIManager *ui_manager;
  GError *tmp_error = NULL;
  guint id;

  g_return_val_if_fail (GTK_IS_WINDOW (window), 0);
  g_return_val_if_fail (filename != NULL, 0);
  g_return_val_if_fail (error == NULL || *error == NULL, 0);

  ui_manager = g_object_get_data (G_OBJECT (window), "__ui_manager__");

  g_return_val_if_fail (GTK_IS_UI_MANAGER (ui_manager), 0);

  if (!(id = gtk_ui_manager_add_ui_from_file (ui_manager, filename, &tmp_error)))
    g_propagate_error (error, tmp_error);

  return id;
}

void
ogmrip_app_add_actions (GtkWindow *window, GtkActionEntry *entries, guint n_entries)
{
  GtkActionGroup *action_group;

  g_return_if_fail (GTK_IS_WINDOW (window));

  action_group = g_object_get_data (G_OBJECT (window), "__action_group__");

  g_return_if_fail (GTK_IS_ACTION_GROUP (action_group));

  gtk_action_group_add_actions (action_group, entries, n_entries, NULL);
}

void
ogmrip_app_set_menus (GtkWindow *window, const gchar *path)
{
  GtkUIManager *ui_manager;
  GtkWidget *widget;

  g_return_if_fail (GTK_IS_WINDOW (window));
  g_return_if_fail (GTK_IS_VBOX (GTK_BIN (window)->child));
  g_return_if_fail (path != NULL);

  ui_manager = g_object_get_data (G_OBJECT (window), "__ui_manager__");

  g_return_if_fail (GTK_IS_UI_MANAGER (ui_manager));

  widget = gtk_ui_manager_get_widget (ui_manager, path);
  gtk_box_pack_start (GTK_BOX (GTK_BIN (window)->child), widget, FALSE, FALSE, 0);
}

GtkWidget *
ogmrip_app_get_widget (GtkWindow *window, const gchar *path)
{
  GtkUIManager *ui_manager;

  g_return_val_if_fail (GTK_IS_WINDOW (window), NULL);
  g_return_val_if_fail (path != NULL, NULL);

  ui_manager = g_object_get_data (G_OBJECT (window), "__ui_manager__");

  g_return_val_if_fail (GTK_IS_UI_MANAGER (ui_manager), NULL);

  return gtk_ui_manager_get_widget (ui_manager, path);
}

static gboolean
ogmrip_app_menu_item_enter (GtkWidget *item, GdkEvent *event, GtkWidget *window)
{
  GtkWidget *statusbar;
  gchar *hint;

  statusbar = g_object_get_data (G_OBJECT (window), "__statusbar__");
  if (statusbar)
  {
    hint = g_object_get_data (G_OBJECT (item), "__menu_hint__");
    if (hint)
      ogmrip_statusbar_push (GTK_STATUSBAR (statusbar), hint);
  }

  return FALSE;
}

static gboolean
ogmrip_app_menu_item_leave (GtkWidget *item, GdkEvent *event, GtkWidget *window)
{
  GtkWidget *statusbar;

  statusbar = g_object_get_data (G_OBJECT (window), "__statusbar__");
  if (statusbar)
    ogmrip_statusbar_pop (GTK_STATUSBAR (statusbar));

  return FALSE;
}

GtkAction *
ogmrip_app_action_connect (GtkWindow *window, const gchar *path, 
    GCallback callback, gpointer user_data, gboolean sensitive, gboolean swapped)
{
  GtkUIManager *ui_manager;
  GtkAction *action;

  g_return_val_if_fail (GTK_IS_WINDOW (window), NULL);
  g_return_val_if_fail (path != NULL, NULL);

  ui_manager = g_object_get_data (G_OBJECT (window), "__ui_manager__");

  g_return_val_if_fail (GTK_IS_UI_MANAGER (ui_manager), NULL);

  action = gtk_ui_manager_get_action (ui_manager, path);
  if (action)
  {
    gchar *hint;

    gtk_action_set_sensitive (action, sensitive);

    if (callback)
    {
      if (swapped)
        g_signal_connect_swapped (action, "activate", callback, user_data);
      else
        g_signal_connect (action, "activate", callback, user_data);
    }

    g_object_get (action, "tooltip", &hint, NULL);
    if (hint)
    {
      GtkWidget *widget;

      widget = gtk_ui_manager_get_widget (ui_manager, path);
      g_object_set_data_full (G_OBJECT (widget), "__menu_hint__", hint, (GDestroyNotify) g_free);

      g_signal_connect (widget, "enter-notify-event", G_CALLBACK (ogmrip_app_menu_item_enter), window);
      g_signal_connect (widget, "leave-notify-event", G_CALLBACK (ogmrip_app_menu_item_leave), window);
    }
  }
  return action;
}

void
ogmrip_app_set_statusbar (GtkWindow *window, GtkWidget *statusbar)
{
  g_return_if_fail (GTK_IS_WINDOW (window));
  g_return_if_fail (GTK_IS_STATUSBAR (statusbar));

  g_object_set_data (G_OBJECT (window), "__statusbar__", statusbar);

  if (!statusbar->parent)
  {
    g_return_if_fail (GTK_IS_VBOX (GTK_BIN (window)->child));

    gtk_box_pack_end (GTK_BOX (GTK_BIN (window)->child), statusbar, FALSE, FALSE, 0);
  }
}

