/* OGMRip - A DVD Encoder for GNOME
 * Copyright (C) 2004-2005 Olivier Rolland <billl@users.sf.fr>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#include "config.h"
#include "ogmrip-pref.h"
#include "ogmrip-progress.h"
#include "ogmrip-helper.h"
#include "ogmrip-gconf.h"
#include "ogmrip-options.h"
#include <ogmrip-dvd-chooser.h>
#include <ogmrip-dvd-dialog.h>

#include <ogmdvd.h>
#include <ogmrip.h>

#ifdef HAVE_X264_SUPPORT
#include <ogmrip-x264.h>
#endif

#ifdef HAVE_MKV_SUPPORT
#include <ogmrip-mkv.h>
#endif

#ifdef HAVE_SRT_SUPPORT
#include <ogmrip-srt.h>
#endif

#ifdef HAVE_ENCHANT_SUPPORT
#include "ogmrip-spell.h"
#endif

#include <sys/stat.h>
#include <glib/gi18n.h>
#include <glib/gstdio.h>

#include <unistd.h>
#include <string.h>
#include <fcntl.h>
#include <math.h>

#define OGMRIP_UI_FILE    "ogmrip/ogmrip-ui.xml"
#define OGMRIP_GLADE_FILE "ogmrip/ogmrip-main.glade"
#define OGMRIP_ICON_FILE  "pixmaps/ogmrip.png"

#define OGMRIP_DEFAULT_FILE_NAME "movie"

#define G_FILE(f)        ((GFile *) (f))
#define GTK_BOX_CHILD(b) ((GtkBoxChild *) (b))

enum
{
  EXTRACT,
  CHAPTER,
  TITLE,
  LENGTH,
  FRAMES,
  COL_LAST
};

enum
{
  OGMRIP_CONTAINER_OGM,
  OGMRIP_CONTAINER_AVI,
  OGMRIP_CONTAINER_MKV
};

enum
{
  OGMRIP_AUDIO_VORBIS,
  OGMRIP_AUDIO_MP3,
  OGMRIP_AUDIO_WAV,
  OGMRIP_AUDIO_COPY
};

enum
{
  OGMRIP_VIDEO_XVID,
  OGMRIP_VIDEO_LAVC,
  OGMRIP_VIDEO_X264,
  OGMRIP_VIDEO_THEORA
};

typedef struct
{
  OGMDvd *dvd;

  GtkWidget *window;

  GtkWidget *pref_dialog;
  GtkWidget *options_dialog;
  GtkWidget *progress_dialog;

  GtkWidget *title_entry;
  GtkWidget *video_combo;
  GtkWidget *length_label;
  GtkWidget *relative_check;

  GtkWidget *audio_vbox;
  GtkWidget *subp_vbox;

  GtkListStore *chapters_store;

  GtkAction *extract_action;
  GtkWidget *extract_button;

  GConfClient *gconf;

  gboolean encoding;

} OGMRipData;

static void ogmrip_main_add_stream_clicked    (OGMRipData *data,
                                               GtkWidget  *button);
static void ogmrip_main_remove_stream_clicked (OGMRipData *data,
                                               GtkWidget  *button);

#ifdef HAVE_ENCHANT_SUPPORT
gboolean
ogmrip_main_spell_check (OGMRipData *data, const gchar *file, const gchar *lang)
{
  gchar *new_file = NULL;
  gchar *text, *corrected;
  gboolean retval = FALSE;

  GtkWidget *checker = NULL;
  GIOChannel *input = NULL, *output = NULL;
  GIOStatus status = G_IO_STATUS_NORMAL;

  checker = ogmrip_spell_new (lang);
  if (!checker)
  {
    ogmrip_message_dialog (GTK_WINDOW (data->window), GTK_MESSAGE_ERROR, 
        _("Could not create dictionary."), _("Spell will not be checked."));
    goto spell_check_cleanup;
  }

  input = g_io_channel_new_file (file, "r", NULL);
  if (!input)
    goto spell_check_cleanup;

  new_file = g_file_make_temp ("sub.XXXXXX", NULL);
  if (new_file < 0)
    goto spell_check_cleanup;

  output = g_io_channel_new_file (new_file, "w", NULL);
  if (!output)
    goto spell_check_cleanup;

  gtk_window_set_parent (GTK_WINDOW (checker), GTK_WINDOW (data->window));
  gtk_widget_show (checker);

  do
  {
    status = g_io_channel_read_line (input, &text, NULL, NULL, NULL);
    if (status == G_IO_STATUS_NORMAL)
    {
      retval = ogmrip_spell_check_text (checker, text, &corrected);
      if (retval)
      {
        do
        {
          status = g_io_channel_write_chars (output, corrected ? corrected : text, -1, NULL, NULL);
        }
        while (status == G_IO_STATUS_AGAIN);

        g_free (corrected);
      }
      g_free (text);
    }
  } 
  while (retval == TRUE && (status == G_IO_STATUS_NORMAL || status == G_IO_STATUS_AGAIN));

  retval &= status == G_IO_STATUS_EOF;

spell_check_cleanup:
  if (checker)
    gtk_widget_destroy (checker);

  if (output)
  {
    g_io_channel_shutdown (output, TRUE, NULL);
    g_io_channel_unref (output);
  }

  if (input)
  {
    g_io_channel_shutdown (input, TRUE, NULL);
    g_io_channel_unref (input);
  }

  if (retval)
    retval = g_file_rename (new_file, file, NULL);
  else
    unlink (new_file);

  g_free (new_file);

  return retval;
}
#endif /* HAVE_ENCHANT_SUPPORT */

/*
 * Loads a DVD disk or a DVD structure
 */
static void
ogmrip_main_load (OGMRipData *data, gboolean device, const gchar *path)
{
  OGMDvd *dvd;
  OGMDvdVideoStream *vstream;
  GtkTreeModel *model;

  model = gtk_combo_box_get_model (GTK_COMBO_BOX (data->video_combo));
  gtk_list_store_clear (GTK_LIST_STORE (model));

  dvd = ogmdvd_open (path);

  if (!dvd)
  {
    if (device)
    {
      if (ogmdvd_tray_open (path))
        ogmrip_message_dialog (GTK_WINDOW (data->window), GTK_MESSAGE_ERROR, 
            _("Could not read the DVD"), _("Tray seems to be opened"));
      else if (ogmdvd_is_valid_media (path))
        ogmrip_message_dialog (GTK_WINDOW (data->window), GTK_MESSAGE_ERROR, 
            _("Could not read the DVD"), _("Device does not contain a valid DVD video"));
      else
        ogmrip_message_dialog (GTK_WINDOW (data->window), GTK_MESSAGE_ERROR, 
            _("Could not read the DVD"), _("Unknown error"));
    }
    else
    {
      ogmrip_message_dialog (GTK_WINDOW (data->window), GTK_MESSAGE_ERROR, 
          _("Could not read the DVD"), _("Path does not contain a valid DVD structure"));
    }
  }
  else
  {
    gchar *str, *title = NULL;
    guint vid, nvid, lvid;
    glong length, longest;
    gint format, aspect;

    if (data->dvd)
      ogmdvd_unref (data->dvd);
    data->dvd = dvd;

    nvid = ogmdvd_get_num_video_streams (dvd);

    for (vid = lvid = 0, longest = 0; vid < nvid; vid++)
    {
      vstream = ogmdvd_get_video_stream (dvd, vid);
      format = ogmdvd_video_stream_get_format (vstream);
      aspect = ogmdvd_video_stream_get_display_aspect (vstream);
      length = ogmdvd_video_stream_get_length (vstream, NULL);

      if (length > longest)
      {
        longest = length;
        lvid = vid;
      }

      str = g_strdup_printf ("%s %02d (%s, %s)", _("Title"), vid + 1,
          ogmdvd_get_video_format_name (format),
          ogmdvd_get_video_aspect_name (aspect));
      gtk_combo_box_append_text (GTK_COMBO_BOX (data->video_combo), str);
      g_free (str);

      ogmdvd_video_stream_unref (vstream);
    }

    if (nvid > 0)
    {
      title = ogmdvd_get_title (dvd);
      gtk_combo_box_set_active (GTK_COMBO_BOX (data->video_combo), lvid);
    }

    gtk_widget_set_sensitive (data->video_combo, nvid > 0);
    gtk_widget_set_sensitive (data->title_entry, nvid > 0);
    gtk_widget_set_sensitive (data->extract_button, nvid > 0);
    gtk_action_set_sensitive (data->extract_action, nvid > 0);

    gtk_entry_set_text (GTK_ENTRY (data->title_entry), title ? title : "");
    g_free (title);
  }
}

static void
ogmrip_main_clear (OGMRipData *data)
{
  GtkTreeModel *model;

  model = gtk_combo_box_get_model (GTK_COMBO_BOX (data->video_combo));
  gtk_list_store_clear (GTK_LIST_STORE (model));

  gtk_combo_box_set_active (GTK_COMBO_BOX (data->video_combo), -1);

  gtk_widget_set_sensitive (data->video_combo, FALSE);
  gtk_widget_set_sensitive (data->title_entry, FALSE);
  gtk_widget_set_sensitive (data->extract_button, FALSE);
  gtk_action_set_sensitive (data->extract_action, FALSE);

  gtk_entry_set_text (GTK_ENTRY (data->title_entry), "");
}

static void
ogmrip_main_fill_audio_combo (GtkWidget *combo, OGMDvdVideoStream *vstream, gint pref)
{
  OGMDvdAudioStream *astream;
  gint channels, format, lang, type;
  gint aid, naid, selected;
  gchar *str;

  naid = ogmdvd_video_stream_get_num_audio_streams (vstream);

  for (aid = selected = 0; aid < naid; aid++)
  {
    astream = ogmdvd_video_stream_get_audio_stream (vstream, aid);

    channels = ogmdvd_audio_stream_get_channels (astream);
    format = ogmdvd_audio_stream_get_format (astream);
    lang = ogmdvd_audio_stream_get_language (astream);
    type = ogmdvd_audio_stream_get_type (astream);

    if (!selected && lang == pref)
      selected = aid;

    if (type > 0)
      str = g_strdup_printf ("%s %02d: %s (%s, %s, %s)", _("Track"), aid + 1,
          ogmdvd_get_audio_type_name (type), ogmdvd_get_language_name (lang),
          ogmdvd_get_audio_format_name (format), ogmdvd_get_audio_channels_name (channels));
    else
      str = g_strdup_printf ("%s %02d: (%s, %s, %s)", _("Track"), aid + 1,
          ogmdvd_get_language_name (lang), ogmdvd_get_audio_format_name (format), 
          ogmdvd_get_audio_channels_name (channels));

    gtk_combo_box_append_text (GTK_COMBO_BOX (combo), str);
    g_free (str);

    ogmdvd_audio_stream_unref (astream);
  }

  gtk_widget_set_sensitive (combo, naid > 0);
  if (naid > 0)
    gtk_combo_box_set_active (GTK_COMBO_BOX (combo), selected);
}

#ifdef HAVE_SRT_SUPPORT
static void
ogmrip_main_fill_subp_combo (GtkWidget *combo, OGMDvdVideoStream *vstream, gint pref)
{
  OGMDvdSubpStream *sstream;
  gint nsid, sid, lang, selected;
  gchar *str;

  gtk_combo_box_append_text (GTK_COMBO_BOX (combo), _("No subtitle"));

  nsid = ogmdvd_video_stream_get_num_subp_streams (vstream);

  for (sid = selected = 0; sid < nsid; sid++)
  {
    sstream = ogmdvd_video_stream_get_subp_stream (vstream, sid);

    lang = ogmdvd_subp_stream_get_language (sstream);

    if (!selected && lang == pref)
      selected = sid;

    str = g_strdup_printf ("%s %02d (%s)", _("Subtitle"), sid + 1,
        ogmdvd_get_language_name (lang));

    gtk_combo_box_append_text (GTK_COMBO_BOX (combo), str);
    g_free (str);

    ogmdvd_subp_stream_unref (sstream);
  }

#ifdef HAVE_SRT_SUPPORT
  gtk_widget_set_sensitive (combo, nsid > 0);
#endif /* HAVE_SRT_SUPPORT */
  if (nsid > 0)
    gtk_combo_box_set_active (GTK_COMBO_BOX (combo), selected);
}
#endif /* HAVE_SRT_SUPPORT */

static void
ogmrip_main_fill_chapters_list (GtkListStore *store, OGMDvdVideoStream *vstream)
{
  GtkTreeIter iter;
  OGMDvdTime time_;
  gint chap, nchap;
  glong frames;
  gchar *str;

  nchap = ogmdvd_video_stream_get_num_chapters (vstream);
  for (chap = 0; chap < nchap; chap++)
  {
    gtk_list_store_append (store, &iter);

    str = g_strdup_printf ("%s %02d", _("Chapter"), chap + 1);
    gtk_list_store_set (store, &iter, EXTRACT, TRUE, CHAPTER, chap + 1, TITLE, str, -1);
    g_free (str);

    if ((frames = ogmdvd_video_stream_get_chapters_length (vstream, &time_, chap, chap)) > 0)
    {
      str = g_strdup_printf ("%02d:%02d:%02d", time_.hour, time_.min, time_.sec);
      gtk_list_store_set (store, &iter, LENGTH, str, FRAMES, frames, -1);
      g_free (str);
    }
  }
}

static void
ogmrip_main_add_stream (OGMRipData *data, GtkWidget *combo, GtkWidget *vbox)
{
  GtkWidget *hbox1, *hbox2;
  GtkWidget *button, *image;
  GtkTooltips *tooltips;

  hbox1 = gtk_hbox_new (FALSE, 6);
  gtk_box_pack_start (GTK_BOX (vbox), hbox1, TRUE, TRUE, 0);
  gtk_widget_show (hbox1);

  gtk_box_pack_start (GTK_BOX (hbox1), combo, TRUE, TRUE, 0);

  hbox2 = gtk_hbox_new (TRUE, 0);
  gtk_box_pack_start (GTK_BOX (hbox1), hbox2, FALSE, FALSE, 0);
  gtk_widget_show (hbox2);

  tooltips = gtk_tooltips_new ();

  button = gtk_button_new ();
  gtk_box_pack_start (GTK_BOX (hbox2), button, TRUE, TRUE, 0);
  gtk_widget_show (button);

  gtk_tooltips_set_tip (tooltips, button, _("Add a stream"), NULL);

  g_signal_connect_swapped (button, "clicked", 
      G_CALLBACK (ogmrip_main_add_stream_clicked), data);

  image = gtk_image_new_from_stock (GTK_STOCK_ADD, GTK_ICON_SIZE_BUTTON);
  gtk_container_add (GTK_CONTAINER (button), image);
  gtk_widget_show (image);

  button = gtk_button_new ();
  gtk_box_pack_start (GTK_BOX (hbox2), button, TRUE, TRUE, 0);

  gtk_tooltips_set_tip (tooltips, button, _("Remove the stream"), NULL);

  g_signal_connect_swapped (button, "clicked", 
      G_CALLBACK (ogmrip_main_remove_stream_clicked), data);

  image = gtk_image_new_from_stock (GTK_STOCK_REMOVE, GTK_ICON_SIZE_BUTTON);
  gtk_container_add (GTK_CONTAINER (button), image);
  gtk_widget_show (image);
}

static void
ogmrip_main_remove_stream (GtkWidget *hbox)
{
  gtk_container_remove (GTK_CONTAINER (hbox->parent), hbox);
}

static gchar *
ogmrip_main_build_filename (OGMRipData *data, const gchar *path, const gchar *title,
    guint container, guint vcodec, guint acodec, gint alang)
{
  gchar *utf8, *filename, *output;
  const gchar *lang;

  static gchar *vcodec_name[] =
  {
    "XviD",
    "Lavc",
    "X264",
    "Theora"
  };

  static gchar *acodec_name[] =
  {
    "Vorbis",
    "Mp3",
    "Wave",
    "Ac3"
  };

  static gchar *extension[] = 
  {
    "ogm",
    "avi",
    "mkv"
  };

  lang = ogmdvd_get_language_name (alang);

  switch (gconf_client_get_int_default (data->gconf, OGMRIP_GCONF_FILENAME, OGMRIP_DEFAULT_FILENAME))
  {
    case 0:
      utf8 = g_strdup_printf ("%s.%s", 
          title, extension[container]);
      break;
    case 1:
      utf8 = g_strdup_printf ("%s - %s.%s", 
          title, lang, extension[container]);
      break;
    case 2:
      utf8 = g_strdup_printf ("%s - %s - %s.%s", 
          title, lang, vcodec_name[vcodec], extension[container]);
      break;
    case 3:
      utf8 = g_strdup_printf ("%s - %s - %s %s.%s", 
          title, lang, vcodec_name[vcodec], acodec_name[acodec], extension[container]);
      break;
    default:
      utf8 = g_strdup_printf ("%s.%s", 
          OGMRIP_DEFAULT_FILE_NAME, extension[container]);
      break;
  }

  filename = g_filename_from_utf8 (utf8, -1, NULL, NULL, NULL);
  g_free (utf8);

  output = g_build_filename (path, filename, NULL);
  g_free (filename);

  return output;
}

/*
 * Checks if container and codecs are compatible
 */
static gboolean
ogmrip_main_check_container (OGMRipData *data, gint container, gint vcocec, gint acodec)
{
  /*
   * AVI cannot contain vorbis streams
   */
  if (container == OGMRIP_CONTAINER_AVI && acodec == OGMRIP_AUDIO_VORBIS)
  {
    ogmrip_message_dialog (GTK_WINDOW (data->window), GTK_MESSAGE_ERROR, "<b>%s</b>\n\n%s",
        _("The video container and the audio codec are incompatible."), _("Please, choose some others."));
    return FALSE;
  }
  return TRUE;
}

/*
 * Check if container can contain multiple streams
 */
static gboolean
ogmrip_main_check_streams (OGMRipData *data, gint container, GList *astreams, GList *sstreams)
{
  gint nstreams;

  if (container == OGMRIP_CONTAINER_AVI)
  {
    nstreams = g_list_length (astreams);
    if (nstreams > 1)
    {
      ogmrip_message_dialog (GTK_WINDOW (data->window), GTK_MESSAGE_ERROR,
          "%s", _("The AVI container does not support multiple audio streams"));
      return FALSE;
    }

    nstreams = g_list_length (sstreams);
    if (nstreams > 0)
    {
      ogmrip_message_dialog (GTK_WINDOW (data->window), GTK_MESSAGE_ERROR,
          "%s", _("The AVI container does not support subtitle streams"));
      return FALSE;
    }
  }
  return TRUE;
}

static gboolean
ogmrip_main_check_path (OGMRipData *data, const gchar *path)
{
  GError *error = NULL;

  if (!g_file_test (path, G_FILE_TEST_IS_DIR))
  {
    gint response;

    if (g_file_test (path, G_FILE_TEST_EXISTS))
    {
      response = ogmrip_message_dialog (GTK_WINDOW (data->window), GTK_MESSAGE_QUESTION,
          _("'%s' exists, but is not a directory.\nDo you want to remove it?"), path);
      if (response == GTK_RESPONSE_NO)
        return FALSE;
      unlink (path);
    }
    else
    {
      response = ogmrip_message_dialog (GTK_WINDOW (data->window), GTK_MESSAGE_QUESTION,
          _("The directory '%s' does not exist.\nDo you want to create it?"), path);
      if (response == GTK_RESPONSE_NO)
        return FALSE;
    }

    if (!g_dir_make (path, 0777, &error))
    {
      ogmrip_message_dialog (GTK_WINDOW (data->window), GTK_MESSAGE_ERROR, error->message);
      g_error_free (error);
      return FALSE;
    }
  }

  if (access (path, W_OK) == -1)
  {
    ogmrip_message_dialog (GTK_WINDOW (data->window), GTK_MESSAGE_ERROR, 
        _("The directory '%s' is not writable."), path);
    return FALSE;
  }

  return TRUE;
}

static gboolean
ogmrip_main_check_output (OGMRipData *data, const gchar *output)
{
  /*
   * TODO What if multiple targets ?
   */
  if (g_file_test (output, G_FILE_TEST_EXISTS))
  {
    gint response;

    response = ogmrip_message_dialog (GTK_WINDOW (data->window), GTK_MESSAGE_QUESTION,
        _("A file called '%s' exists.\nDo you want to overwrite it?"), output);

    if (response == GTK_RESPONSE_NO)
      return FALSE;

    unlink (output);
  }

  return TRUE;
}

/*
 * Checks space left on device
 */
static gboolean
ogmrip_main_check_space (OGMRipData *data, gint64 rip_size, gint64 dvd_size, const gchar *output)
{
  gchar *output_mp, *tmp_mp;
  gboolean retval = FALSE;

  output_mp = g_fs_get_mount_point (output);
  tmp_mp = g_fs_get_mount_point (ogmrip_get_tmp_dir ());

  if (output_mp && tmp_mp)
  {
    if (strcmp (tmp_mp, output_mp) == 0)
    {
      retval = rip_size * 2 + dvd_size < g_fs_space_left (output);
      if (!retval)
        ogmrip_message_dialog (GTK_WINDOW (data->window), GTK_MESSAGE_ERROR,
            _("Not enough space to store output and temporary files (%ldMB needed)."), (2 * rip_size + dvd_size) / 1024 / 1024);
    }
    else
    {
      retval = rip_size + dvd_size < g_fs_space_left (ogmrip_get_tmp_dir ());
      if (!retval)
        ogmrip_message_dialog (GTK_WINDOW (data->window), GTK_MESSAGE_ERROR,
            _("Not enough space to store the temporary files (%ldMB needed)."), (rip_size + dvd_size) / 1024 / 1024);
      else
      {
        retval = rip_size < g_fs_space_left (output);
        if (!retval)
          ogmrip_message_dialog (GTK_WINDOW (data->window), GTK_MESSAGE_ERROR,
              _("Not enough space to store the output file (%ldMB needed)."), rip_size / 1024 / 1024);
      }
    }
  }

  g_free (output_mp);
  g_free (tmp_mp);

  return retval;
}

static OGMDvdVideoStream *
ogmrip_main_get_video_stream (OGMRipData *data, OGMDvd *dvd)
{
  gint id;
  id = gtk_combo_box_get_active (GTK_COMBO_BOX (data->video_combo));
  return ogmdvd_get_video_stream (dvd, id);
}

static GList *
ogmrip_main_get_audio_streams (OGMRipData *data, OGMDvdVideoStream *vstream)
{
  OGMDvdAudioStream *astream;
  GList *streams, *child;
  GtkWidget *hbox, *combo;
  gint aid;

  streams = NULL;

  for (child = GTK_BOX (data->audio_vbox)->children; child; child = child->next)
  {
    hbox = GTK_BOX_CHILD (child->data)->widget;
    if (GTK_WIDGET_SENSITIVE (hbox))
    {
      combo = gtk_box_get_child (GTK_BOX (hbox), 0);

      aid = gtk_combo_box_get_active (GTK_COMBO_BOX (combo));
      if (aid > -1)
      {
        astream = ogmdvd_video_stream_get_audio_stream (vstream, aid);
        if (astream && !g_list_find (streams, astream))
          streams = g_list_append (streams, astream);
      }
    }
  }

  return streams;
}

#ifdef HAVE_SRT_SUPPORT
static GList *
ogmrip_main_get_subp_streams (OGMRipData *data, OGMDvdVideoStream *vstream)
{
  OGMDvdSubpStream *sstream;
  GList *streams, *child;
  GtkWidget *hbox, *combo;
  gint sid;

  if (!GTK_WIDGET_SENSITIVE (data->subp_vbox))
    return NULL;

  streams = NULL;

  for (child = GTK_BOX (data->subp_vbox)->children; child; child = child->next)
  {
    hbox = GTK_BOX_CHILD (child->data)->widget;
    if (GTK_WIDGET_SENSITIVE (hbox))
    {
      combo = gtk_box_get_child (GTK_BOX (hbox), 0);

      sid = gtk_combo_box_get_active (GTK_COMBO_BOX (combo));
      if (sid > 0)
      {
        sstream = ogmdvd_video_stream_get_subp_stream (vstream, sid - 1);
        if (sstream && !g_list_find (streams, sstream))
          streams = g_list_append (streams, sstream);
      }
    }
  }

  return streams;
}
#endif /* HAVE_SRT_SUPPORT */

static void
ogmrip_main_get_chapter_range (OGMRipData *data, gint *start_chapter, gint *end_chapter)
{
  GtkTreeIter iter;
  gboolean extract, valid;

  valid = gtk_tree_model_get_iter_first (GTK_TREE_MODEL (data->chapters_store), &iter);

  extract = *start_chapter = 0;
  while (valid && !extract)
  {
    gtk_tree_model_get (GTK_TREE_MODEL (data->chapters_store), &iter, EXTRACT, &extract, -1);
    valid = gtk_tree_model_iter_next (GTK_TREE_MODEL (data->chapters_store), &iter);
    if (valid && !extract)
      (*start_chapter) ++;
  }

  *end_chapter = *start_chapter;
  if (!valid && !extract)
    *start_chapter = -1;

  while (valid && extract)
  {
    gtk_tree_model_get (GTK_TREE_MODEL (data->chapters_store), &iter, EXTRACT, &extract, -1);
    valid = gtk_tree_model_iter_next (GTK_TREE_MODEL (data->chapters_store), &iter);
    if (valid && extract)
      (*end_chapter) ++;
  }

  if (!valid && extract)
    *end_chapter = -1;
}

static gchar *
ogmrip_main_get_path (OGMRipData *data)
{
  gchar *utf8, *path;

  utf8 = gconf_client_get_string_default (data->gconf, OGMRIP_GCONF_OUTPUT_DIR, OGMRIP_DEFAULT_OUTPUT_DIR);
  path = g_filename_from_utf8 (utf8, -1, NULL, NULL, NULL);
  g_free (utf8);

  return path;
}

/*
 * Returns the estimated rip size
 */
static gulong
ogmrip_main_get_rip_length (OGMRipData *data, OGMDvdVideoStream *vstream, gint start_chap, gint end_chap)
{
  if (start_chap == 0 && end_chap == -1)
    return ogmdvd_video_stream_get_length (vstream, NULL);
  else
    return ogmdvd_video_stream_get_chapters_length (vstream, NULL, start_chap, end_chap);
}

/*
 * Returns the estimated rip size
 */
static gint64
ogmrip_main_get_rip_size (OGMRipData *data, OGMDvdVideoStream *vstream, 
    gint start_chap, gint end_chap, gint tsize, gint tnumber)
{
  gboolean relative;
  gdouble factor = 1.0;

  relative = GTK_WIDGET_SENSITIVE (data->relative_check) & 
    gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->relative_check));

  if (relative)
  {
    glong chap_len, full_len;

    full_len = ogmrip_main_get_rip_length (data, vstream, 0, -1);
    chap_len = ogmrip_main_get_rip_length (data, vstream, start_chap, end_chap);
    factor = chap_len / (gdouble) full_len;
  }

  return ceil (factor * tsize * tnumber * 1024 * 1024);
}

/*
 * Returns the VMG + VTS size
 */
static gint64
ogmrip_main_get_dvd_size (OGMRipData *data, OGMDvdVideoStream *vstream, gboolean copy_dvd)
{
  if (copy_dvd)
  {
    OGMDvd *dvd;
    gint64 vmg_size, vts_size;

    dvd = ogmdvd_video_stream_get_dvd (vstream);
    if ((vmg_size = ogmdvd_get_vmg_size (dvd)) < 0)
      return -1;

    if ((vts_size = ogmdvd_video_stream_get_vts_size (vstream)) < 0)
      return -1;

    return vmg_size + vts_size;
  }

  return 0;
}

/*
 * Copy the stream on the hard drive
 */
static gint
ogmrip_main_copy_dvd (OGMRipData *data, OGMDvdVideoStream *vstream, const gchar *path, GError **error)
{
  OGMSpSpawn *spawn;
  GError *tmp_error = NULL;
  gint result;

  if (!(spawn = ogmrip_dvdcpy_new (vstream, path)))
    return OGMSP_RESULT_ERROR;

  ogmrip_progress_set_spawn (data->progress_dialog, spawn, _("<b>DVD backup</b>"));
  g_object_unref (spawn);

  result = ogmsp_spawn_run (spawn, &tmp_error);
  if (tmp_error)
    g_propagate_error (error, tmp_error);

  return result;
}

/*
 * Extracts sstream from start_chap to end_chap to output
 */
#ifdef HAVE_SRT_SUPPORT
static gint
ogmrip_main_extract_subp (OGMRipData *data, OGMDvdSubpStream *sstream, 
    gint start_chap, gint end_chap, const gchar *output, GError **error)
{
  OGMSpSpawn *spawn;
  GError *tmp_error = NULL;
  gchar *message;
  gint result;

  if (!(spawn = ogmrip_srt_new (sstream, output)))
    return OGMSP_RESULT_ERROR;

  ogmrip_codec_set_chapters (OGMRIP_CODEC (spawn), start_chap, end_chap);

  message = g_strdup_printf (_("<b>Extracting subtitles stream %d</b>"), ogmdvd_subp_stream_get_id (sstream) + 1);
  ogmrip_progress_set_spawn (data->progress_dialog, spawn, message);
  g_object_unref (spawn);

  result = ogmsp_spawn_run (spawn, &tmp_error);
  if (tmp_error)
    g_propagate_error (error, tmp_error);
  g_free (message);

  return result;
}
#endif /* HAVE_SRT_SUPPORT */

/*
 * Converts astream to acodec from start_chap to end_chap to output
 */
static gint
ogmrip_main_extract_audio (OGMRipData *data, OGMDvdAudioStream *astream, 
    gint acodec, gint start_chap, gint end_chap, const gchar *output, GError **error)
{
  OGMSpSpawn *spawn = NULL;
  GError *tmp_error = NULL;
  gchar *message;
  gint result;

  switch (acodec)
  {
    case OGMRIP_AUDIO_VORBIS:
      spawn = ogmrip_vorbis_new (astream, output);
      break;
    case OGMRIP_AUDIO_MP3:
      spawn = ogmrip_mp3_new (astream, output);
      break;
    case OGMRIP_AUDIO_WAV:
      spawn = ogmrip_wav_new (astream, output);
      break;
    case OGMRIP_AUDIO_COPY:
      spawn = ogmrip_audio_copy_new (astream, output);
      break;
    default:
      break;
  }

  if (!spawn)
    return OGMSP_RESULT_ERROR;

  ogmrip_audio_set_quality (OGMRIP_AUDIO (spawn),
      gconf_client_get_int_default (data->gconf, OGMRIP_GCONF_QUALITY, OGMRIP_DEFAULT_QUALITY));
  ogmrip_audio_set_normalize (OGMRIP_AUDIO (spawn),
      gconf_client_get_bool_default (data->gconf, OGMRIP_GCONF_NORMALIZE, OGMRIP_DEFAULT_NORMALIZE));
  ogmrip_codec_set_chapters (OGMRIP_CODEC (spawn), start_chap, end_chap);

  message = g_strdup_printf (_("<b>Encoding audio stream %d</b>"), ogmdvd_audio_stream_get_id (astream) + 1);
  ogmrip_progress_set_spawn (data->progress_dialog, spawn, message);
  g_object_unref (spawn);

  result = ogmsp_spawn_run (spawn, &tmp_error);
  if (tmp_error)
    g_propagate_error (error, tmp_error);
  g_free (message);
  
  return result;
}

/*
 * Converts vstream to vcodec from start_chap to end_chap to output
 */
static gint
ogmrip_main_extract_video (OGMRipData *data, OGMDvdVideoStream *vstream, gint codec, gint start_chap, gint end_chap, 
    gint64 nonvideo_size, gint64 overhead_size, gint64 total_size, const gchar *output, GError **error)
{
  OGMSpSpawn *spawn = NULL;
  guint x, y, width, height, bitrate;
  GError *tmp_error = NULL;
  gchar *message;
  gint result;

  switch (codec)
  {
    case OGMRIP_VIDEO_XVID:
      spawn = ogmrip_xvid_new (vstream, output);
      break;
    case OGMRIP_VIDEO_LAVC:
      spawn = ogmrip_lavc_new (vstream, output);
      break;
#ifdef HAVE_X264_SUPPORT
    case OGMRIP_VIDEO_X264:
      spawn = ogmrip_x264_new (vstream, output);
      break;
#endif /* HAVE_X264_SUPPORT */
#ifdef HAVE_THEORA_SUPPORT
    case OGMRIP_VIDEO_THEORA:
      break;
#endif /* HAVE_THEORA_SUPPORT */
    default:
      break;
  }

  if (!spawn)
    return OGMSP_RESULT_ERROR;

  ogmrip_video_set_denoise (OGMRIP_VIDEO (spawn),
      gconf_client_get_bool_default (data->gconf, OGMRIP_GCONF_DENOISE, OGMRIP_DEFAULT_DENOISE));
  ogmrip_video_set_trellis (OGMRIP_VIDEO (spawn),
      gconf_client_get_bool_default (data->gconf, OGMRIP_GCONF_TRELLIS, OGMRIP_DEFAULT_TRELLIS));
  ogmrip_video_set_turbo (OGMRIP_VIDEO (spawn),
      gconf_client_get_bool_default (data->gconf, OGMRIP_GCONF_TURBO, OGMRIP_DEFAULT_TURBO));
  ogmrip_video_set_qpel (OGMRIP_VIDEO (spawn),
      gconf_client_get_bool_default (data->gconf, OGMRIP_GCONF_QPEL, OGMRIP_DEFAULT_QPEL));
  ogmrip_video_set_scaler (OGMRIP_VIDEO (spawn),
      gconf_client_get_int_default (data->gconf, OGMRIP_GCONF_SCALER, OGMRIP_DEFAULT_SCALER));
  ogmrip_codec_set_chapters (OGMRIP_CODEC (spawn), start_chap, end_chap);

  ogmrip_progress_set_spawn (data->progress_dialog, spawn, _("<b>Calculating bitrate, cropping, scaling</b>"));
  g_object_unref (spawn);

  ogmrip_video_set_framestep (OGMRIP_VIDEO (spawn), ogmrip_options_get_framestep (data->options_dialog));
  ogmrip_video_set_deinterlacer (OGMRIP_VIDEO (spawn), ogmrip_options_get_deinterlacer (data->options_dialog));
  ogmrip_video_set_pullup (OGMRIP_VIDEO (spawn), ogmrip_options_get_pullup (data->options_dialog));
    
  if (ogmrip_options_get_progressive (data->options_dialog))
    ogmrip_video_set_framerate (OGMRIP_VIDEO (spawn), 24000, 1001);

  if (ogmrip_options_get_bitrate (data->options_dialog, &bitrate) == OGMRIP_OPTIONS_AUTOMATIC)
    ogmrip_video_autobitrate (OGMRIP_VIDEO (spawn), nonvideo_size, overhead_size, total_size);
  else
    ogmrip_video_set_bitrate (OGMRIP_VIDEO (spawn), bitrate);

  if (ogmrip_options_get_crop (data->options_dialog, &x, &y, &width, &height) == OGMRIP_OPTIONS_AUTOMATIC)
    ogmrip_video_autocrop (OGMRIP_VIDEO (spawn), 0);
  else
    ogmrip_video_set_crop_size (OGMRIP_VIDEO (spawn), x, y, width, height);

  if (ogmrip_options_get_scale (data->options_dialog, &width, &height) == OGMRIP_OPTIONS_AUTOMATIC)
    ogmrip_video_autoscale (OGMRIP_VIDEO (spawn));
  else
    ogmrip_video_set_scale_size (OGMRIP_VIDEO (spawn), width, height);

  if (gconf_client_get_int_default (data->gconf, OGMRIP_GCONF_PASSES, OGMRIP_DEFAULT_PASSES) == 0)
  {
    /* Pass 0 */
    message = g_strdup_printf (_("<b>Encoding video title %d</b>"), 
        ogmdvd_video_stream_get_id (vstream) + 1);
    ogmrip_progress_set_spawn (data->progress_dialog, spawn, message);
    result = ogmsp_spawn_run (spawn, &tmp_error);
    if (tmp_error)
      g_propagate_error (error, tmp_error);
    g_free (message);
  }
  else
  {
    const gchar *lfile;
/*
#if MPLAYER_PRE < 8
*/
    /*
     * Workaround against xvid pass log file
     * Should disappear someday
     */
    if (codec == OGMRIP_VIDEO_XVID)
      lfile = g_build_filename (g_get_tmp_dir (), "xvid-twopass.stats", NULL);
    else
/*
#endif
*/
      lfile = g_file_make_temp ("log.XXXXXX", NULL);

    if (!lfile)
      return OGMSP_RESULT_ERROR;

    ogmrip_video_set_log (OGMRIP_VIDEO (spawn), lfile);

    /* Pass 1 */
    ogmrip_video_set_pass (OGMRIP_VIDEO (spawn), 1);
    message = g_strdup_printf (_("<b>Encoding video title %d, Pass 1</b>"), 
        ogmdvd_video_stream_get_id (vstream) + 1);
    ogmrip_progress_set_spawn (data->progress_dialog, spawn, message);

    switch ((result = ogmsp_spawn_run (spawn, &tmp_error)))
    {
      case OGMSP_RESULT_COMPLETED:
        /* Pass 2 */
        g_free (message);
        ogmrip_video_set_pass (OGMRIP_VIDEO (spawn), 2);
        message = g_strdup_printf (_("<b>Encoding video title %d, Pass 2</b>"), 
            ogmdvd_video_stream_get_id (vstream) + 1);
        ogmrip_progress_set_spawn (data->progress_dialog, spawn, message);
        result = ogmsp_spawn_run (spawn, &tmp_error);
        break;
      case OGMSP_RESULT_ERROR:
        break;
      default:
        break;
    }

    if (tmp_error)
      g_propagate_error (error, tmp_error);
    g_free (message);

    g_unlink (lfile);
  }

  return result;
}

/*
 * Extracts chapters of vstream from start_chap to end_chap to output
 */
static gint
ogmrip_main_extract_chapters (OGMRipData *data, OGMDvdVideoStream *vstream, 
    gint start_chap, gint end_chap, const gchar *output, GError **error)
{
  GtkTreeIter iter;

  if (gtk_tree_model_iter_nth_child (GTK_TREE_MODEL (data->chapters_store), &iter, NULL, start_chap))
  {
    GIOChannel *channel;
    guint i, numerator, denominator;
    gulong frames, length;
    gchar *str, *title;

    channel = g_io_channel_new_file (output, "w", NULL);
    gtk_tree_model_get (GTK_TREE_MODEL (data->chapters_store), &iter, 
        FRAMES, &frames, TITLE, &title, -1);

    str = g_strdup_printf ("CHAPTER01NAME=%s\n", title);
    g_io_channel_write_chars (channel, "CHAPTER01=00:00:00.000\n", -1, NULL, NULL);
    g_io_channel_write_chars (channel, str, -1, NULL, NULL);
    g_free (title);
    g_free (str);

    ogmdvd_video_stream_get_framerate (vstream, &numerator, &denominator);
    if (end_chap == -1)
      end_chap = ogmdvd_video_stream_get_num_chapters (vstream);

    for (i = start_chap, length = 0; i < end_chap; i++)
    {
      if (gtk_tree_model_iter_next (GTK_TREE_MODEL (data->chapters_store), &iter))
      {
        length += (gulong) (frames * denominator * 1000 / numerator);
        str = g_strdup_printf ("CHAPTER%02d=%02lu:%02lu:%02lu.%03lu\n", i - start_chap + 2, 
            length / (60 * 60 * 1000), length / (60 * 1000) % 60, length / 1000 % 60, length % 1000);
        g_io_channel_write_chars (channel, str, -1, NULL, NULL);
        g_free (str);

        gtk_tree_model_get (GTK_TREE_MODEL (data->chapters_store), &iter, 
            FRAMES, &frames, TITLE, &title, -1);

        str = g_strdup_printf ("CHAPTER%02dNAME=%s\n", i - start_chap + 2, title);
        g_io_channel_write_chars (channel, str, -1, NULL, NULL);
        g_free (title);
        g_free (str);
      }
    }

    g_io_channel_shutdown (channel, TRUE, NULL);
    g_io_channel_unref (channel);

    return OGMSP_RESULT_COMPLETED;
  }

  return OGMSP_RESULT_ERROR;
}

/*
 * Merges (and split) vfile, afile, sfile and cfile in container to output
 */
static gint
ogmrip_main_merge (OGMRipData *data, OGMSpSpawn *container, GError **error)
{
  GError *tmp_error = NULL;
  gint result;
/*
#if MPLAYER_PRE < 8
*/
  gchar *cwd;
/*
#endif
*/
  ogmrip_progress_set_spawn (data->progress_dialog, container, _("<b>Merging audio and video streams</b>"));
/*
#if MPLAYER_PRE < 8
*/
  cwd = g_get_current_dir ();
  chdir (ogmrip_get_tmp_dir ());
/*
#endif
*/
  result = ogmsp_spawn_run (container, &tmp_error);
  if (tmp_error)
    g_propagate_error (error, tmp_error);
/*
#if MPLAYER_PRE < 8
*/
  chdir (cwd);
  g_free (cwd);
/*
#endif
*/
  return result;
}

static void
ogmrip_main_progress_dialog_construct (OGMRipData *data, guint steps)
{
  data->progress_dialog = ogmrip_progress_new (gtk_entry_get_text (GTK_ENTRY (data->title_entry)), steps);
  g_signal_connect (data->progress_dialog, "destroy", G_CALLBACK (gtk_widget_destroyed), &data->progress_dialog);
  gtk_window_set_parent (GTK_WINDOW (data->progress_dialog), GTK_WINDOW (data->window));
}

/*
 * Events
 */

static gboolean
ogmrip_main_delete_event (OGMRipData *data)
{
  if (data->encoding)
    return TRUE;

  return FALSE;
}

static void
ogmrip_main_destroyed (OGMRipData *data)
{
  if (data->gconf)
    g_object_unref (data->gconf);

  if (data->dvd)
    ogmdvd_unref (data->dvd);

  if (data->pref_dialog)
    gtk_widget_destroy (data->pref_dialog);

  g_object_unref (data->chapters_store);
  g_free (data);
}

static void
ogmrip_main_extract_activated (OGMRipData *data)
{
  if (!data->progress_dialog && data->dvd)
  {
    GError *error = NULL;
    GList *audio_streams, *subp_streams, *stream;
    OGMDvdVideoStream *video_stream;
    OGMSpSpawn *container;

    gint result = OGMSP_RESULT_COMPLETED;

    gboolean copy_dvd, keep_temp;;
    gint tsize, tnumber, passes, vcodec, acodec;
    gint64 nvsize, ohsize, rip_size, dvd_size;
    gint start_chap, end_chap, cont;
    gchar *path, *output, *filename;
    const gchar *title;
    glong rip_length;

    data->encoding = TRUE;

    copy_dvd = FALSE;
    filename = output = NULL;
    container = NULL;

    title = gtk_entry_get_text (GTK_ENTRY (data->title_entry));

    video_stream = ogmrip_main_get_video_stream (data, data->dvd);
    audio_streams = ogmrip_main_get_audio_streams (data, video_stream);
#ifdef HAVE_SRT_SUPPORT
    subp_streams = ogmrip_main_get_subp_streams (data, video_stream);
#else /* HAVE_SRT_SUPPORT */
    subp_streams = NULL;
#endif /* HAVE_SRT_SUPPORT */

    ogmrip_main_get_chapter_range (data, &start_chap, &end_chap);

    keep_temp = gconf_client_get_bool_default (data->gconf, OGMRIP_GCONF_KEEP_TEMP, OGMRIP_DEFAULT_KEEP_TEMP);

    cont = gconf_client_get_int_default (data->gconf, OGMRIP_GCONF_CONTAINER, OGMRIP_DEFAULT_CONTAINER);
    vcodec = gconf_client_get_int_default (data->gconf, OGMRIP_GCONF_VIDEO, OGMRIP_DEFAULT_VIDEO);
    acodec = gconf_client_get_int_default (data->gconf, OGMRIP_GCONF_AUDIO, OGMRIP_DEFAULT_AUDIO);

    tnumber = gconf_client_get_int_default (data->gconf, OGMRIP_GCONF_TNUMBER, OGMRIP_DEFAULT_TNUMBER);
    tsize = gconf_client_get_int_default (data->gconf, OGMRIP_GCONF_TSIZE, OGMRIP_DEFAULT_TSIZE);

    data->options_dialog = ogmrip_options_new (video_stream);
    gtk_window_set_parent (GTK_WINDOW (data->options_dialog), GTK_WINDOW (data->window));
    if (gtk_dialog_run (GTK_DIALOG (data->options_dialog)) != GTK_RESPONSE_OK)
      goto extract_cleanup;

    /*
     * Check if container and codecs are compatible
     */
    if (!ogmrip_main_check_container (data, cont, vcodec, acodec))
      goto extract_cleanup;

    /*
     * Check if container can contain multiple streams
     */
    if (!ogmrip_main_check_streams (data, cont, audio_streams, subp_streams))
      goto extract_cleanup;

    path = ogmrip_main_get_path (data);

    /*
     * Check output path
     */
    if (!ogmrip_main_check_path (data, path))
    {
      g_free (path);
      goto extract_cleanup;
    }

    /*
     * Build filename
     */
    output = ogmrip_main_build_filename (data, path, title, cont, vcodec, acodec, 
        ogmdvd_audio_stream_get_language (audio_streams->data));
    g_free (path);

    /*
     * Check output file
     */
    if (!ogmrip_main_check_output (data, output))
    {
      g_free (output);
      goto extract_cleanup;
    }

    copy_dvd = gconf_client_get_bool_default (data->gconf, OGMRIP_GCONF_COPY_DVD, OGMRIP_DEFAULT_COPY_DVD);
    if (copy_dvd)
    {
      const gchar *device;
      struct stat buf;

      device = ogmdvd_get_device (ogmdvd_video_stream_get_dvd (video_stream));
      if (stat (device, &buf) < 0)
        goto extract_cleanup;

      copy_dvd = S_ISBLK (buf.st_mode);
    }

    rip_length = ogmrip_main_get_rip_length (data, video_stream, start_chap, end_chap);
    if (rip_length < 0)
      goto extract_cleanup;

    rip_size = ogmrip_main_get_rip_size (data, video_stream, start_chap, end_chap, tsize, tnumber);
    if (rip_size < 0)
      goto extract_cleanup;

    dvd_size = ogmrip_main_get_dvd_size (data, video_stream, copy_dvd);
    if (dvd_size < 0)
      goto extract_cleanup;

    /*
     * Check space left on device
     */

    if (!ogmrip_main_check_space (data, rip_size, dvd_size, output))
      goto extract_cleanup;

    passes = gconf_client_get_int_default (data->gconf, OGMRIP_GCONF_PASSES, OGMRIP_DEFAULT_PASSES) + 
      3 + (tnumber > 1 ? 1 : 0) + g_list_length (audio_streams) + g_list_length (subp_streams);
    ogmrip_main_progress_dialog_construct (data, passes);
    gtk_widget_show (data->progress_dialog);

    /*
     * If DVD copy
     */
    if (copy_dvd)
    {
      OGMDvd *dvd;
      gchar *path;

      path = g_dir_make_temp ("dvdXXXXXX", 0755, NULL);
      if (!path)
        goto extract_cleanup;

      if ((result = ogmrip_main_copy_dvd (data, video_stream, path, &error)) != OGMSP_RESULT_COMPLETED)
        goto extract_cleanup;

      dvd = ogmdvd_open (path);
      g_free (path);

      if (!dvd)
        goto extract_cleanup;

      ogmdvd_video_stream_unref (video_stream);
      video_stream = ogmrip_main_get_video_stream (data, dvd);

      g_list_foreach (audio_streams, (GFunc) ogmdvd_audio_stream_unref, NULL);
      audio_streams = ogmrip_main_get_audio_streams (data, video_stream);

#ifdef HAVE_SRT_SUPPORT
      g_list_foreach (subp_streams, (GFunc) ogmdvd_subp_stream_unref, NULL);
      subp_streams = ogmrip_main_get_subp_streams (data, video_stream);
#endif /* HAVE_SRT_SUPPORT */

      ogmdvd_unref (dvd);
    }

    /*
     * Container
     */
    switch (cont)
    {
      case OGMRIP_CONTAINER_AVI:
        container = ogmrip_avi_new (output);
        break;
#ifdef HAVE_MKV_SUPPORT
      case OGMRIP_CONTAINER_MKV:
        container = ogmrip_matroska_new (output);
        break;
#endif /* HAVE_MKV_SUPPORT */
      default:
        container = ogmrip_ogg_new (output);
        break;
    }

    if (!container)
      goto extract_cleanup;

    
    ogmrip_container_set_unlink_on_unref (OGMRIP_CONTAINER (container), !keep_temp);

    ogmrip_container_set_target_size (OGMRIP_CONTAINER (container), tnumber, tsize);
    ogmrip_container_set_title (OGMRIP_CONTAINER (container), title);

    /*
     * Chapters extraction
     */
    filename = g_file_make_temp ("chapters.XXXXXX", &error);
    if (!filename)
      goto extract_cleanup;

    if ((result = ogmrip_main_extract_chapters (data, video_stream, start_chap, end_chap, filename, &error)) != OGMSP_RESULT_COMPLETED)
      goto extract_cleanup;

    ogmrip_container_add_chapters (OGMRIP_CONTAINER (container), filename, -1);

    g_free (filename); filename = NULL;

#ifdef HAVE_SRT_SUPPORT
    /*
     * Subtitles Extraction
     */
    for (stream = subp_streams; stream; stream = stream->next)
    {
      gint lang;

      filename = g_file_make_temp ("sub.XXXXXX", &error);
      if (!filename)
        goto extract_cleanup;

      if ((result = ogmrip_main_extract_subp (data, stream->data, start_chap, end_chap, filename, &error)) != OGMSP_RESULT_COMPLETED)
        goto extract_cleanup;

      lang = ogmdvd_subp_stream_get_language (stream->data);

      ogmrip_container_add_subp (OGMRIP_CONTAINER (container), filename, lang);

#ifdef HAVE_ENCHANT_SUPPORT
      if (gconf_client_get_bool_default (data->gconf, OGMRIP_GCONF_SPELL_CHECK, OGMRIP_DEFAULT_SPELL_CHECK))
        ogmrip_main_spell_check (data, filename, ogmdvd_get_language_iso639_1 (lang));
#endif /* HAVE_ENCHANT_SUPPORT */

      g_free (filename); filename = NULL;
    }
#endif /* HAVE_SRT_SUPPORT */

    /*
     * Audio encoding
     */
    if (!gconf_client_get_bool_default (data->gconf, OGMRIP_GCONF_NOAUDIO, OGMRIP_DEFAULT_NOAUDIO))
    {
      for (stream = audio_streams; stream; stream = stream->next)
      {
        if (cont == OGMRIP_CONTAINER_AVI && acodec == OGMRIP_AUDIO_COPY)
          filename = g_file_make_temp ("frameno.XXXXXX", &error);
        else
          filename = g_file_make_temp ("audio.XXXXXX", &error);

        if (!filename)
          goto extract_cleanup;

        if ((result = ogmrip_main_extract_audio (data, stream->data, acodec, start_chap, end_chap, filename, &error)) != OGMSP_RESULT_COMPLETED)
          goto extract_cleanup;

        ogmrip_container_add_audio (OGMRIP_CONTAINER (container), filename, ogmdvd_audio_stream_get_language (stream->data));

        g_free (filename); filename = NULL;
      }
    }

    /*
     * Video encoding
     */
    filename = g_file_make_temp ("video.XXXXXX", &error);
    if (!filename)
      goto extract_cleanup;

    /*
     * Non video size for auto bitrate calculation
     */
    nvsize = ogmrip_container_get_nonvideo_size (OGMRIP_CONTAINER (container));
    ohsize = ogmrip_container_get_overhead_size (OGMRIP_CONTAINER (container), rip_size, rip_length);

#ifdef OGMRIP_DEBUG
    g_message ("Non video size: %" G_GINT64_FORMAT, nvsize);
    g_message ("Overhead size: %" G_GINT64_FORMAT, ohsize);
#endif /* OGMRIP_DEBUG */

    if ((result = ogmrip_main_extract_video (data, video_stream, vcodec, start_chap, end_chap, nvsize, 
          ohsize, rip_size, filename, &error)) != OGMSP_RESULT_COMPLETED)
      goto extract_cleanup;

    ogmrip_container_set_video (OGMRIP_CONTAINER (container), filename);

    g_free (filename); filename = NULL;

    /*
     * Merge
     */
    if ((result = ogmrip_main_merge (data, container, &error)) != OGMSP_RESULT_COMPLETED)
      goto extract_cleanup;

extract_cleanup:
    if (container)
      g_object_unref (container);

    if (data->progress_dialog)
      gtk_widget_destroy (data->progress_dialog);

    if (data->options_dialog)
      gtk_widget_destroy (data->options_dialog);
    data->options_dialog = NULL;

    g_list_foreach (audio_streams, 
        (GFunc) ogmdvd_audio_stream_unref, NULL);
    g_list_free (audio_streams);

    g_list_foreach (subp_streams, 
        (GFunc) ogmdvd_subp_stream_unref, NULL);
    g_list_free (subp_streams);

    if (video_stream)
    {
      if (copy_dvd)
      {
        path = g_strdup (ogmdvd_get_device (ogmdvd_video_stream_get_dvd (video_stream)));

        if (!keep_temp && path && g_file_test (path, G_FILE_TEST_IS_DIR))
          g_dir_remove (path, TRUE, NULL);
        g_free (path);
      }
      ogmdvd_video_stream_unref (video_stream);
    }

    if (result == OGMSP_RESULT_ERROR || error != NULL)
    {
      if (error)
        ogmrip_message_dialog (GTK_WINDOW (data->window), GTK_MESSAGE_ERROR, error->message);
      else
        ogmrip_message_dialog (GTK_WINDOW (data->window), GTK_MESSAGE_ERROR, _("Unknown error"));
    }

    if (filename && g_file_test (filename, G_FILE_TEST_EXISTS))
      g_unlink (filename);
    g_free (filename);

    if (error)
      g_error_free (error);

    g_file_unref (output, tnumber > 1);

    data->encoding = FALSE;
  }
}

static void
ogmrip_main_eject_activated (OGMRipData *data, GtkWidget *dialog)
{
  const NautilusBurnDrive *drive;
  const gchar *device;

  drive = ogmrip_dvd_dialog_get_drive (OGMRIP_DVD_DIALOG (dialog));
  if (data->dvd)
  {
    device = ogmdvd_get_device (data->dvd);
    if (strcmp (device, drive->device) == 0)
    {
      ogmdvd_unref (data->dvd);
      data->dvd = NULL;

      ogmrip_main_clear (data);
    }
  }
}

static void
ogmrip_main_load_activated (OGMRipData *data)
{
  if (!data->encoding)
  {
    GtkWidget *dialog;

    dialog = ogmrip_dvd_dialog_new ();
    gtk_window_set_icon_from_stock (GTK_WINDOW (dialog), GTK_STOCK_REFRESH);
    gtk_window_set_parent (GTK_WINDOW (dialog), GTK_WINDOW (data->window));

    g_signal_connect_swapped (dialog, "eject", G_CALLBACK (ogmrip_main_eject_activated), data);

    if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_OK)
    {
      const NautilusBurnDrive *drive;

      drive = ogmrip_dvd_dialog_get_drive (OGMRIP_DVD_DIALOG (dialog));
      if (drive)
        ogmrip_main_load (data, TRUE, drive->device);
    }
    gtk_widget_destroy (dialog);
  }
}

static void
ogmrip_main_open_activated (OGMRipData *data)
{
  if (!data->encoding)
  {
    GtkWidget *dialog;

    dialog = gtk_file_chooser_dialog_new (_("Open DVD Structure"), 
        GTK_WINDOW (data->window), GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER, 
        GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL, GTK_STOCK_OPEN, GTK_RESPONSE_OK, NULL);
    gtk_window_set_icon_from_stock (GTK_WINDOW (dialog), GTK_STOCK_OPEN);
    gtk_window_set_parent (GTK_WINDOW (dialog), GTK_WINDOW (data->window));

    if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_OK)
    {
      gchar *dir;

      dir = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dialog));
      if (dir)
        ogmrip_main_load (data, FALSE, dir);
    }
    gtk_widget_destroy (dialog);
  }
}

static void
ogmrip_main_select_all_activated (OGMRipData *data)
{
  OGMDvdVideoStream *vstream;
  OGMDvdTime time_;

  GtkTreeModel *model;
  GtkTreeIter iter;

  model = GTK_TREE_MODEL (data->chapters_store);
  if (gtk_tree_model_get_iter_first (model, &iter))
  {
    do
    {
      gtk_list_store_set (data->chapters_store, &iter, EXTRACT, TRUE, -1);
    }
    while (gtk_tree_model_iter_next (model, &iter));
  }

  vstream = ogmrip_main_get_video_stream (data, data->dvd);
  if (ogmdvd_video_stream_get_length (vstream, &time_) > 0)
  {
    gchar *str;

    str = g_strdup_printf ("%02d:%02d:%02d", time_.hour, time_.min, time_.sec);
    gtk_label_set_text (GTK_LABEL (data->length_label), str);
    g_free (str);
  }
  ogmdvd_video_stream_unref (vstream);

  gtk_widget_set_sensitive (data->extract_button, TRUE);
  gtk_widget_set_sensitive (data->relative_check, FALSE);
  gtk_action_set_sensitive (data->extract_action, TRUE);
}

static void
ogmrip_main_deselect_all_activated (OGMRipData *data)
{
  GtkTreeModel *model;
  GtkTreeIter iter;

  model = GTK_TREE_MODEL (data->chapters_store);
  if (gtk_tree_model_get_iter_first (model, &iter))
  {
    do
    {
      gtk_list_store_set (data->chapters_store, &iter, EXTRACT, FALSE, -1);
    }
    while (gtk_tree_model_iter_next (model, &iter));
  }

  gtk_label_set_text (GTK_LABEL (data->length_label), "");
  gtk_widget_set_sensitive (data->extract_button, FALSE);
  gtk_widget_set_sensitive (data->relative_check, FALSE);
  gtk_action_set_sensitive (data->extract_action, FALSE);
}

static void
ogmrip_main_pref_activated (OGMRipData *data)
{
  gtk_widget_show (data->pref_dialog);
}

static void
ogmrip_main_about_activated (OGMRipData *data)
{
  static GdkPixbuf *icon = NULL;

  const gchar *authors[] = 
  { 
    "Olivier Rolland <billl@users.sf.net>", 
    NULL 
  };
  gchar *translator_credits = _("translator-credits");

  const gchar *documenters[] = 
  { 
    "Olivier Brisson <olivier.brisson@gmail.com>",
    NULL 
  };

  if (!icon)
    icon = gdk_pixbuf_new_from_file (OGMRIP_DATA_DIR "/" OGMRIP_ICON_FILE, NULL);

  if (strcmp (translator_credits, "translator-credits") == 0)
    translator_credits = NULL;

  gtk_show_about_dialog (GTK_WINDOW (data->window), 
      "name", PACKAGE_NAME,
      "version", PACKAGE_VERSION,
      "comments", _("A DVD Encoder for GNOME"),
      "copyright", "(c) 2004-2005 Olivier Rolland",
      "website", "http://ogmrip.sourceforge.net",
      "translator-credits", translator_credits,
      "documenters", documenters,
      "authors", authors,
      "logo", icon,
      NULL);
}

static void
ogmrip_main_video_combo_changed (OGMRipData *data)
{
  GtkWidget *audio_combo;
#ifdef HAVE_SRT_SUPPORT
  GtkWidget *subp_combo;
  GtkWidget *hbox;
#endif /* HAVE_SRT_SUPPORT */

  OGMDvdVideoStream *vstream;
  OGMDvdTime time_;
  gchar *str;

  gtk_list_store_clear (data->chapters_store);

  gtk_container_foreach (GTK_CONTAINER (data->audio_vbox), 
      (GtkCallback) ogmrip_main_remove_stream, NULL);

  audio_combo = gtk_combo_box_new_text ();
  ogmrip_main_add_stream (data, audio_combo, data->audio_vbox);
  gtk_widget_show (audio_combo);

  gtk_widget_set_sensitive (data->audio_vbox, data->dvd != NULL);

#ifdef HAVE_SRT_SUPPORT
  gtk_container_foreach (GTK_CONTAINER (data->subp_vbox), 
      (GtkCallback) ogmrip_main_remove_stream, NULL);

  subp_combo = gtk_combo_box_new_text ();
  ogmrip_main_add_stream (data, subp_combo, data->subp_vbox);
  gtk_widget_show (subp_combo);

  gtk_widget_set_sensitive (data->subp_vbox, data->dvd != NULL);
#endif /* HAVE_SRT_SUPPORT */

  gtk_label_set_text (GTK_LABEL (data->length_label), "");

  if (data->dvd)
  {
    gint container, pref;

    vstream = ogmrip_main_get_video_stream (data, data->dvd);

    if (ogmdvd_video_stream_get_length (vstream, &time_) > 0)
    {
      str = g_strdup_printf ("%02d:%02d:%02d", time_.hour, time_.min, time_.sec);
      gtk_label_set_text (GTK_LABEL (data->length_label), str);
      g_free (str);
    }

    container = gconf_client_get_int_default (data->gconf, 
        OGMRIP_GCONF_CONTAINER, OGMRIP_DEFAULT_CONTAINER);

    pref = gconf_client_get_int_default (data->gconf, 
        OGMRIP_GCONF_PREF_AUDIO, OGMRIP_DEFAULT_PREF_AUDIO);
    ogmrip_main_fill_audio_combo (audio_combo, vstream, pref);

#ifdef HAVE_SRT_SUPPORT
    pref = gconf_client_get_int_default (data->gconf, 
        OGMRIP_GCONF_PREF_SUBP, OGMRIP_DEFAULT_PREF_SUBP);
    ogmrip_main_fill_subp_combo (subp_combo, vstream, pref);

    gtk_widget_set_sensitive (data->subp_vbox, container != OGMRIP_CONTAINER_AVI);
    hbox = gtk_box_get_child (GTK_BOX (data->audio_vbox), 0);
    gtk_widget_set_sensitive (gtk_box_get_child (GTK_BOX (hbox), 1), 
        container != OGMRIP_CONTAINER_AVI);
#endif /* HAVE_SRT_SUPPORT */

    ogmrip_main_fill_chapters_list (data->chapters_store, vstream);

    ogmdvd_video_stream_unref (vstream);

    gtk_widget_set_sensitive (data->relative_check, FALSE);
  }
}

static void
ogmrip_main_add_stream_clicked (OGMRipData *data, GtkWidget *button)
{
  GtkWidget *vbox, *combo;
  OGMDvdVideoStream *vstream;

  vbox = button->parent->parent->parent;
  gtk_widget_hide (button);

  button = gtk_box_get_child (GTK_BOX (button->parent), 1);
  gtk_widget_show (button);

  combo = gtk_combo_box_new_text ();
  ogmrip_main_add_stream (data, combo, vbox);
  gtk_widget_show (combo);

  vstream = ogmrip_main_get_video_stream (data, data->dvd);

  if (vbox == data->audio_vbox)
    ogmrip_main_fill_audio_combo (combo, vstream, 0);
#ifdef HAVE_SRT_SUPPORT
  else
    ogmrip_main_fill_subp_combo (combo, vstream, 0);
#endif /* HAVE_SRT_SUPPORT */

  ogmdvd_video_stream_unref (vstream);
}

static void
ogmrip_main_remove_stream_clicked (OGMRipData *data, GtkWidget *button)
{
  ogmrip_main_remove_stream (button->parent->parent);
}

static void
ogmrip_main_extract_cell_toggled (OGMRipData *data, gchar *path)
{
  GtkTreeIter iter, next_iter, prev_iter;
  gboolean extract, next_extract, prev_extract;
  gint start_chap, end_chap;

  OGMDvdVideoStream *vstream;
  OGMDvdTime time_;
  gchar *str;

  next_extract = prev_extract = FALSE;

  gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (data->chapters_store), &iter, path);

  next_iter = iter;
  if (gtk_tree_model_iter_next (GTK_TREE_MODEL (data->chapters_store), &next_iter))
    gtk_tree_model_get (GTK_TREE_MODEL (data->chapters_store), &next_iter, EXTRACT, &next_extract, -1);

  prev_iter = iter;
  if (gtk_tree_model_iter_prev (GTK_TREE_MODEL (data->chapters_store), &prev_iter))
    gtk_tree_model_get (GTK_TREE_MODEL (data->chapters_store), &prev_iter, EXTRACT, &prev_extract, -1);

  if ((prev_extract && next_extract) || (!prev_extract && !next_extract))
    ogmrip_main_deselect_all_activated (data);

  gtk_tree_model_get (GTK_TREE_MODEL (data->chapters_store), &iter, EXTRACT, &extract, -1);
  gtk_list_store_set (data->chapters_store, &iter, EXTRACT, extract ? FALSE : TRUE, -1);

  ogmrip_main_get_chapter_range (data, &start_chap, &end_chap);

  vstream = ogmrip_main_get_video_stream (data, data->dvd);
  if (ogmdvd_video_stream_get_chapters_length (vstream, &time_, start_chap, end_chap) > 0)
  {
    str = g_strdup_printf ("%02d:%02d:%02d", time_.hour, time_.min, time_.sec);
    gtk_label_set_text (GTK_LABEL (data->length_label), str);
    g_free (str);
  }
  ogmdvd_video_stream_unref (vstream);

  gtk_widget_set_sensitive (data->extract_button, start_chap != -1);
  gtk_action_set_sensitive (data->extract_action, start_chap != -1);

  gtk_widget_set_sensitive (data->relative_check, start_chap > 0 || end_chap != -1);
}

static void
ogmrip_main_title_cell_edited (OGMRipData *data, gchar *path, gchar *text)
{
  GtkTreeIter iter;

  gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (data->chapters_store), &iter, path);
  gtk_list_store_set (data->chapters_store, &iter, TITLE, text, -1);
}

static void
ogmrip_main_container_notified (GConfClient *gconf, guint id, GConfEntry *entry, OGMRipData *data)
{
#ifdef HAVE_SRT_SUPPORT
  gboolean sensitive;

  if (data->dvd)
  {
    GtkWidget *hbox;
    guint i = 0;

    sensitive = gconf_value_get_int (entry->value) != OGMRIP_CONTAINER_AVI;
    gtk_widget_set_sensitive (data->subp_vbox, sensitive);

    while ((hbox = gtk_box_get_child (GTK_BOX (data->audio_vbox), i ++)))
    {
      if (i > 1)
        gtk_widget_set_sensitive (hbox, sensitive);
      else
        gtk_widget_set_sensitive (gtk_box_get_child (GTK_BOX (hbox), 1), sensitive);
    }
  }
#endif /* HAVE_SRT_SUPPORT */
}

static OGMRipData *
ogmrip_main_new (void)
{
  GtkActionEntry action_entries[] =
  {
    { "FileMenu",    NULL,                  N_("_File"),         NULL,          NULL,                           NULL },
    { "Load",        GTK_STOCK_CDROM,       N_("_Load"),         "<ctl>L",      N_("Load a DVD disk"),          NULL },
    { "Open",        GTK_STOCK_OPEN,        NULL,                NULL,          N_("Open a DVD structure"),     NULL },
    { "Extract",     GTK_STOCK_CONVERT,     N_("E_xtract"),      "<ctl>Return", N_("Extract selected streams"), NULL },
    { "Quit",        GTK_STOCK_QUIT,        NULL,                NULL,          N_("Exit OGMRip"),              NULL },
    { "EditMenu",    NULL,                  N_("_Edit"),         NULL,          NULL,                           NULL },
    { "SelectAll",   NULL,                  N_("Select _All"),   "<ctl>A",      N_("Select all chapters"),      NULL },
    { "DeselectAll", NULL,                  N_("_Deselect All"), "<ctl>D",      N_("Deselect all chapters"),    NULL },
    { "Preferences", GTK_STOCK_PREFERENCES, NULL,                NULL,          N_("Edit the preferences"),     NULL },
    { "HelpMenu",    NULL,                  N_("_Help"),         NULL,          NULL,                           NULL },
    { "About",       GTK_STOCK_ABOUT,       N_("_About"),        NULL,          N_("About OGMRip"),             NULL },
  };

  OGMRipData *data;
  GtkWidget *widget;
  GladeXML *xml;

  GtkTreeViewColumn *column;
  GtkCellRenderer *renderer;

  xml = glade_xml_new (OGMRIP_DATA_DIR "/" OGMRIP_GLADE_FILE, NULL, NULL);
  if (!xml)
  {
    g_warning ("Could not find " OGMRIP_GLADE_FILE);
    return NULL;
  }

  data = g_new0 (OGMRipData, 1);

  data->gconf = gconf_client_get_default ();
  gconf_client_add_dir (data->gconf, OGMRIP_GCONF_ROOT, 
      GCONF_CLIENT_PRELOAD_ONELEVEL, NULL);

  gconf_client_notify_add (data->gconf, OGMRIP_GCONF_CONTAINER, 
      (GConfClientNotifyFunc) ogmrip_main_container_notified, data, NULL, NULL);

  data->window = glade_xml_get_widget (xml, "main-window");
  gtk_window_set_default_size (GTK_WINDOW (data->window), 350, 500);
  gtk_window_set_icon_from_file (GTK_WINDOW (data->window), OGMRIP_DATA_DIR "/" OGMRIP_ICON_FILE, NULL);

  g_signal_connect_swapped (data->window, "delete-event", G_CALLBACK (ogmrip_main_delete_event), data);
  g_signal_connect_swapped (data->window, "destroy", G_CALLBACK (ogmrip_main_destroyed), data);
  g_signal_connect (data->window, "destroy", G_CALLBACK (gtk_main_quit), NULL);

  ogmrip_app_init (GTK_WINDOW (data->window), GETTEXT_PACKAGE);
  ogmrip_app_add_ui_from_file (GTK_WINDOW (data->window), OGMRIP_DATA_DIR "/" OGMRIP_UI_FILE, NULL);
  ogmrip_app_add_actions (GTK_WINDOW (data->window), action_entries, G_N_ELEMENTS (action_entries));
  ogmrip_app_set_statusbar (GTK_WINDOW (data->window), glade_xml_get_widget (xml, "statusbar"));
  ogmrip_app_set_menus (GTK_WINDOW (data->window), "/Menubar");

  data->extract_action = ogmrip_app_action_connect (GTK_WINDOW (data->window), "/Menubar/FileMenu/Extract", 
      G_CALLBACK (ogmrip_main_extract_activated), data, FALSE, TRUE);

  ogmrip_app_action_connect (GTK_WINDOW (data->window), "/Menubar/FileMenu/Load", 
      G_CALLBACK (ogmrip_main_load_activated), data, TRUE, TRUE);
  ogmrip_app_action_connect (GTK_WINDOW (data->window), "/Menubar/FileMenu/Open", 
      G_CALLBACK (ogmrip_main_open_activated), data, TRUE, TRUE);
  ogmrip_app_action_connect (GTK_WINDOW (data->window), "/Menubar/FileMenu/Quit", 
      G_CALLBACK (gtk_widget_destroy), data->window, TRUE, TRUE);
  ogmrip_app_action_connect (GTK_WINDOW (data->window), "/Menubar/EditMenu/Preferences", 
      G_CALLBACK (ogmrip_main_pref_activated), data, TRUE, TRUE);
  ogmrip_app_action_connect (GTK_WINDOW (data->window), "/Menubar/EditMenu/SelectAll", 
      G_CALLBACK (ogmrip_main_select_all_activated), data, TRUE, TRUE);
  ogmrip_app_action_connect (GTK_WINDOW (data->window), "/Menubar/EditMenu/DeselectAll", 
      G_CALLBACK (ogmrip_main_deselect_all_activated), data, TRUE, TRUE);
  ogmrip_app_action_connect (GTK_WINDOW (data->window), "/Menubar/HelpMenu/About", 
      G_CALLBACK (ogmrip_main_about_activated), data, TRUE, TRUE);

  widget = glade_xml_get_widget (xml, "load-button");
  g_signal_connect_swapped (widget, "clicked", G_CALLBACK (ogmrip_main_load_activated), data);

  widget = glade_xml_get_widget (xml, "open-button");
  g_signal_connect_swapped (widget, "clicked", G_CALLBACK (ogmrip_main_open_activated), data);

  widget = data->extract_button = glade_xml_get_widget (xml, "extract-button");
  g_signal_connect_swapped (widget, "clicked", G_CALLBACK (ogmrip_main_extract_activated), data);

  widget = data->video_combo = glade_xml_get_widget (xml, "video-combo");
  g_signal_connect_swapped (widget, "changed", G_CALLBACK (ogmrip_main_video_combo_changed), data);

  data->subp_vbox = glade_xml_get_widget (xml, "subp-vbox");
  data->audio_vbox = glade_xml_get_widget (xml, "audio-vbox");
  data->length_label = glade_xml_get_widget (xml, "length-label");
  data->relative_check = glade_xml_get_widget (xml, "relative-check");
  data->title_entry = glade_xml_get_widget (xml, "title-entry");

  widget = glade_xml_get_widget (xml, "treeview");

  data->chapters_store = gtk_list_store_new (COL_LAST, 
      G_TYPE_BOOLEAN, G_TYPE_UINT, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_ULONG);
  gtk_tree_view_set_model (GTK_TREE_VIEW (widget), GTK_TREE_MODEL (data->chapters_store));

  renderer = gtk_cell_renderer_toggle_new ();
  column = gtk_tree_view_column_new_with_attributes (_("Extract?"), renderer, "active", EXTRACT, NULL);
  gtk_tree_view_append_column (GTK_TREE_VIEW (widget), column);
  g_object_set (renderer, "activatable", TRUE, NULL);

  g_signal_connect_swapped (renderer, "toggled", G_CALLBACK (ogmrip_main_extract_cell_toggled), data);

  renderer = gtk_cell_renderer_text_new ();
  column = gtk_tree_view_column_new_with_attributes (_("Chapter"), renderer, "text", CHAPTER, NULL);
  gtk_tree_view_append_column (GTK_TREE_VIEW (widget), column);

  renderer = gtk_cell_renderer_text_new ();
  column = gtk_tree_view_column_new_with_attributes (_("Duration"), renderer, "text", LENGTH, NULL);
  gtk_tree_view_append_column (GTK_TREE_VIEW (widget), column);

  renderer = gtk_cell_renderer_text_new ();
  column = gtk_tree_view_column_new_with_attributes (_("Title"), renderer, "text", TITLE, NULL);
  gtk_tree_view_append_column (GTK_TREE_VIEW (widget), column);
  gtk_tree_view_column_set_resizable (column, TRUE);
  g_object_set (renderer, "editable", TRUE, NULL);

  g_signal_connect_swapped (renderer, "edited", G_CALLBACK (ogmrip_main_title_cell_edited), data);

  g_object_unref (xml);

  return data;
}

int
main (int argc, char *argv[])
{
  OGMRipData *data;

#ifdef ENABLE_NLS
  bindtextdomain (GETTEXT_PACKAGE, LOCALEDIR);
  bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
  textdomain (GETTEXT_PACKAGE);
#endif /* ENABLE_NLS */

  gtk_init (&argc, &argv);

  data = ogmrip_main_new ();

  data->pref_dialog = ogmrip_pref_new ();
  gtk_window_set_parent (GTK_WINDOW (data->pref_dialog), GTK_WINDOW (data->window));

  gtk_widget_show (data->window);

  gtk_main ();

  return 0;
}

