/* OGMRip - A DVD Encoder for GNOME
 * Copyright (C) 2004-2005 Olivier Rolland <billl@users.sf.fr>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#include "config.h"

#include "ogmdvd.h"

#include "ogmrip-pref.h"
#include "ogmrip-gconf.h"
#include "ogmrip-helper.h"

#include <string.h>
#include <glib/gi18n.h>
#include <glade/glade.h>

#define OGMRIP_GLADE_FILE "ogmrip/ogmrip-pref.glade"

enum
{
  OGMRIP_CONF_CHOOSER,
  OGMRIP_CONF_COMBO,
  OGMRIP_CONF_SPIN,
  OGMRIP_CONF_TOGGLE,
  OGMRIP_CONF_RADIO,
  OGMRIP_CONF_LANG
};

typedef struct
{
  gint  type;
  const gchar *name;
  const gchar *key;
  gint value;
  GtkWidget *widget;
} OGMRipConf;

extern gchar *ogmdvd_languages[][3];
extern guint ogmdvd_nlanguages;

static void
ogmrip_pref_set_active_lang (GtkComboBox *combo, gint code)
{
  gint i = 0;

  if (code)
  {
    const gchar *lang;

    lang = ogmdvd_get_language_iso639_1 (code);
    for (i = 1; i < ogmdvd_nlanguages; i++)
      if (strcmp (ogmdvd_languages[i + 1][OGMDVD_LANGUAGE_ISO639_1], lang) == 0)
        break;

    if (i == ogmdvd_nlanguages)
      i = 0;
  }

  gtk_combo_box_set_active (GTK_COMBO_BOX (combo), i);
}

static gint
ogmrip_pref_get_active_lang (GtkComboBox *combo)
{
  const gchar *lang;
  gint i;

  i = gtk_combo_box_get_active (combo);
  if (i > 0 && i + 1 < ogmdvd_nlanguages)
  {
    lang = ogmdvd_languages[i + 1][OGMDVD_LANGUAGE_ISO639_1];
    return (lang[0] << 8) | lang[1];
  }

  return 0;
}

/*
 * Signals
 */

static void
ogmrip_pref_chooser_changed (GtkWidget *button, GConfClient *client)
{
  gchar *key;

  if ((key = g_object_get_data (G_OBJECT (button), "__gconf_key__")))
  {
    gchar *path;

    path = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (button));
    gconf_client_set_string (client, key, path, NULL);
    g_free (path);
  }
}

static void
ogmrip_pref_combo_changed (GtkWidget *combo, GConfClient *client)
{
  gchar *key;

  if ((key = g_object_get_data (G_OBJECT (combo), "__gconf_key__")))
    gconf_client_set_int (client, key, gtk_combo_box_get_active (GTK_COMBO_BOX (combo)), NULL);
}

static void
ogmrip_pref_spin_changed (GtkWidget *spin, GConfClient *client)
{
  gchar *key;

  if ((key = g_object_get_data (G_OBJECT (spin), "__gconf_key__")))
    gconf_client_set_int (client, key, gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (spin)), NULL);
}

static void
ogmrip_pref_toggle_changed (GtkWidget *toggle, GConfClient *client)
{
  gchar *key;

  if ((key = g_object_get_data (G_OBJECT (toggle), "__gconf_key__")))
    gconf_client_set_bool (client, key, gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (toggle)), NULL);
}

static void
ogmrip_pref_radio_changed (GtkWidget *radio, GConfClient *client)
{
  if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (radio)))
  {
    gchar *key;

    if ((key = g_object_get_data (G_OBJECT (radio), "__gconf_key__")))
      gconf_client_set_int (client, key, gtk_radio_button_get_active (GTK_RADIO_BUTTON (radio)), NULL);
  }
}

static void
ogmrip_pref_lang_changed (GtkWidget *combo, GConfClient *client)
{
  gchar *key;

  if ((key = g_object_get_data (G_OBJECT (combo), "__gconf_key__")))
    gconf_client_set_int (client, key, ogmrip_pref_get_active_lang (GTK_COMBO_BOX (combo)), NULL);
}

/*
 * Notifications
 */

static void
ogmrip_pref_combo_notified (GConfClient *client, guint id, GConfEntry *entry, GtkComboBox *combo)
{
  GtkTreeModel *model;

  model = gtk_combo_box_get_model (combo);
  gtk_combo_box_set_active (combo, 
      CLAMP (gconf_value_get_int (entry->value), 0, gtk_tree_model_iter_n_children (model, NULL) - 1));
}

static void
ogmrip_pref_spin_notified (GConfClient *client, guint id, GConfEntry *entry, GtkSpinButton *spin)
{
  gtk_spin_button_set_value (spin, 
      CLAMP (gconf_value_get_int (entry->value), spin->adjustment->lower, spin->adjustment->upper));
}

static void
ogmrip_pref_toggle_notified (GConfClient *client, guint id, GConfEntry *entry, GtkToggleButton *toggle)
{
  gtk_toggle_button_set_active (toggle, gconf_value_get_bool (entry->value));
}

static void
ogmrip_pref_radio_notified (GConfClient *client, guint id, GConfEntry *entry, GtkRadioButton *radio)
{
  GSList *group;

  group = gtk_radio_button_get_group (radio);
  gtk_radio_button_set_active (radio, 
      CLAMP (gconf_value_get_int (entry->value), 0, g_slist_length (group) - 1));
}

static void
ogmrip_pref_lang_notified (GConfClient *client, guint id, GConfEntry *entry, GtkComboBox *combo)
{
  GtkTreeModel *model;

  model = gtk_combo_box_get_model (combo);
  ogmrip_pref_set_active_lang (combo, 
      CLAMP (gconf_value_get_int (entry->value), 0, gtk_tree_model_iter_n_children (model, NULL) - 1));
}

static void
ogmrip_pref_output_dir_notified (GConfClient *client, guint id, GConfEntry *entry, GtkWidget *chooser)
{
  if (entry->value == NULL)
    gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (chooser), g_get_home_dir());
  else
  {
    const gchar *value;
    gchar *path;

    value = gconf_value_get_string (entry->value);
    path = gtk_file_chooser_get_current_folder (GTK_FILE_CHOOSER (chooser));

    if (strcmp (value, path) != 0)
      gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (chooser), value);

    g_free (path);
  }
}

static void
ogmrip_pref_tmp_dir_notified (GConfClient *client, guint id, GConfEntry *entry, GtkWidget *chooser)
{
  if (entry->value == NULL)
    gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (chooser), g_get_home_dir());
  else
  {
    const gchar *value;
    gchar *path;

    value = gconf_value_get_string (entry->value);
    path = gtk_file_chooser_get_current_folder (GTK_FILE_CHOOSER (chooser));

    if (strcmp (value, path) != 0)
    {
      gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (chooser), value);
      ogmrip_set_tmp_dir (value);
    }

    g_free (path);
  }
}

static void
ogmrip_pref_init (GladeXML *xml, GConfClient *client, OGMRipConf *conf)
{
  GSList *group;
  guint i;

  for (i = 0; conf[i].type != -1; i++)
  {
    conf[i].widget = glade_xml_get_widget (xml, conf[i].name);

    if (conf[i].key)
      g_object_set_data (G_OBJECT (conf[i].widget), "__gconf_key__", (gpointer) conf[i].key);

    switch (conf[i].type)
    {
      case OGMRIP_CONF_CHOOSER:
        g_signal_connect (conf[i].widget, "selection-changed", G_CALLBACK (ogmrip_pref_chooser_changed), client);
        break;
      case OGMRIP_CONF_COMBO:
        gtk_combo_box_set_active (GTK_COMBO_BOX (conf[i].widget),
            gconf_client_get_int_default (client, conf[i].key, conf[i].value));
        g_signal_connect (conf[i].widget, "changed", G_CALLBACK (ogmrip_pref_combo_changed), client);
        gconf_client_notify_add (client, conf[i].key, 
            (GConfClientNotifyFunc) ogmrip_pref_combo_notified, conf[i].widget, NULL, NULL);
        break;
      case OGMRIP_CONF_SPIN:
        gtk_spin_button_set_value (GTK_SPIN_BUTTON (conf[i].widget), 
            gconf_client_get_int_default (client, conf[i].key, conf[i].value));
        g_signal_connect (conf[i].widget, "value-changed", G_CALLBACK (ogmrip_pref_spin_changed), client);
        gconf_client_notify_add (client, conf[i].key, 
            (GConfClientNotifyFunc) ogmrip_pref_spin_notified, conf[i].widget, NULL, NULL);
        break;
      case OGMRIP_CONF_TOGGLE:
        gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (conf[i].widget),
            gconf_client_get_bool_default (client, conf[i].key, conf[i].value));
        g_signal_connect (conf[i].widget, "toggled", G_CALLBACK (ogmrip_pref_toggle_changed), client);
        gconf_client_notify_add (client, conf[i].key, 
            (GConfClientNotifyFunc) ogmrip_pref_toggle_notified, conf[i].widget, NULL, NULL);
        break;
      case OGMRIP_CONF_RADIO:
        group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (conf[i].widget));
        g_signal_connect (conf[i].widget, "toggled", G_CALLBACK (ogmrip_pref_radio_changed), client);
        if (g_slist_index (group, conf[i].widget) == gconf_client_get_int_default (client, conf[i].key, conf[i].value))
        {
          gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (conf[i].widget), TRUE);
          gconf_client_notify_add (client, conf[i].key, 
              (GConfClientNotifyFunc) ogmrip_pref_radio_notified, conf[i].widget, NULL, NULL);
        }
        break;
      case OGMRIP_CONF_LANG:
        ogmrip_pref_set_active_lang (GTK_COMBO_BOX (conf[i].widget),
            gconf_client_get_int_default (client, conf[i].key, conf[i].value));
        g_signal_connect (conf[i].widget, "changed", G_CALLBACK (ogmrip_pref_lang_changed), client);
        gconf_client_notify_add (client, conf[i].key, 
            (GConfClientNotifyFunc) ogmrip_pref_lang_notified, conf[i].widget, NULL, NULL);
        break;
      default:
        g_assert_not_reached ();
    }
  }
}

GtkWidget *
ogmrip_pref_new (void)
{
  OGMRipConf pref_conf[] = 
  {
    { OGMRIP_CONF_CHOOSER, "output-dir-chooser", OGMRIP_GCONF_OUTPUT_DIR,  0,                           NULL },
    { OGMRIP_CONF_CHOOSER, "tmp-dir-chooser",    OGMRIP_GCONF_TMP_DIR,     0,                           NULL },
    { OGMRIP_CONF_COMBO,   "filename-combo",     OGMRIP_GCONF_FILENAME,    OGMRIP_DEFAULT_FILENAME,     NULL },
    { OGMRIP_CONF_COMBO,   "container-combo",    OGMRIP_GCONF_CONTAINER,   OGMRIP_DEFAULT_CONTAINER,    NULL },
    { OGMRIP_CONF_SPIN,    "tnumber-spin",       OGMRIP_GCONF_TNUMBER,     OGMRIP_DEFAULT_TNUMBER,      NULL },
    { OGMRIP_CONF_SPIN,    "tsize-spin",         OGMRIP_GCONF_TSIZE,       OGMRIP_DEFAULT_TSIZE,        NULL },
    { OGMRIP_CONF_TOGGLE,  "copy-dvd-check",     OGMRIP_GCONF_COPY_DVD,    OGMRIP_DEFAULT_COPY_DVD,     NULL },
    { OGMRIP_CONF_TOGGLE,  "keep-tmp-check",     OGMRIP_GCONF_KEEP_TEMP,   OGMRIP_DEFAULT_KEEP_TEMP,    NULL },
    { OGMRIP_CONF_RADIO,   "xvid-radio",         OGMRIP_GCONF_VIDEO,       OGMRIP_DEFAULT_VIDEO,        NULL },
    { OGMRIP_CONF_RADIO,   "lavc-radio",         OGMRIP_GCONF_VIDEO,       OGMRIP_DEFAULT_VIDEO,        NULL },
    { OGMRIP_CONF_RADIO,   "x264-radio",         OGMRIP_GCONF_VIDEO,       OGMRIP_DEFAULT_VIDEO,        NULL },
    { OGMRIP_CONF_RADIO,   "theora-radio",       OGMRIP_GCONF_VIDEO,       OGMRIP_DEFAULT_VIDEO,        NULL },
    { OGMRIP_CONF_COMBO,   "passes-combo",       OGMRIP_GCONF_PASSES,      OGMRIP_DEFAULT_PASSES,       NULL },
    { OGMRIP_CONF_COMBO,   "scaler-combo",       OGMRIP_GCONF_SCALER,      OGMRIP_DEFAULT_SCALER,       NULL },
    { OGMRIP_CONF_TOGGLE,  "trellis-check",      OGMRIP_GCONF_TRELLIS,     OGMRIP_DEFAULT_TRELLIS,      NULL },
    { OGMRIP_CONF_TOGGLE,  "qpel-check",         OGMRIP_GCONF_QPEL,        OGMRIP_DEFAULT_QPEL,         NULL },
    { OGMRIP_CONF_TOGGLE,  "turbo-check",        OGMRIP_GCONF_TURBO,       OGMRIP_DEFAULT_TURBO,        NULL },
    { OGMRIP_CONF_TOGGLE,  "denoise-check",      OGMRIP_GCONF_DENOISE,     OGMRIP_DEFAULT_DENOISE,      NULL },
    { OGMRIP_CONF_LANG,    "pref-audio-combo",   OGMRIP_GCONF_PREF_AUDIO,  OGMRIP_DEFAULT_PREF_AUDIO,   NULL },
    { OGMRIP_CONF_RADIO,   "vorbis-radio",       OGMRIP_GCONF_AUDIO,       OGMRIP_DEFAULT_AUDIO,        NULL },
    { OGMRIP_CONF_RADIO,   "mp3-radio",          OGMRIP_GCONF_AUDIO,       OGMRIP_DEFAULT_AUDIO,        NULL },
    { OGMRIP_CONF_RADIO,   "wave-radio",         OGMRIP_GCONF_AUDIO,       OGMRIP_DEFAULT_AUDIO,        NULL },
    { OGMRIP_CONF_RADIO,   "ac3-radio",          OGMRIP_GCONF_AUDIO,       OGMRIP_DEFAULT_AUDIO,        NULL },
    { OGMRIP_CONF_SPIN,    "quality-spin",       OGMRIP_GCONF_QUALITY,     OGMRIP_DEFAULT_QUALITY,      NULL },
    { OGMRIP_CONF_TOGGLE,  "normalize-check",    OGMRIP_GCONF_NORMALIZE,   OGMRIP_DEFAULT_NORMALIZE,    NULL },
    { OGMRIP_CONF_LANG,    "pref-subp-combo",    OGMRIP_GCONF_PREF_SUBP,   OGMRIP_DEFAULT_PREF_SUBP,    NULL },
    { OGMRIP_CONF_TOGGLE,  "spell-check",        OGMRIP_GCONF_SPELL_CHECK, OGMRIP_DEFAULT_SPELL_CHECK,  NULL },
    { -1,                  NULL,                 NULL,                     -1,                          NULL },
  };

  GConfClient *client;
  GtkWidget *dialog;
  GtkWidget *widget;
  GladeXML *xml;
  gchar *str;
  guint i;

  xml = glade_xml_new (OGMRIP_DATA_DIR "/" OGMRIP_GLADE_FILE, NULL, NULL);
  if (!xml)
  {
    g_warning ("Could not find " OGMRIP_GLADE_FILE);
    return NULL;
  }

  client = gconf_client_get_default ();

  dialog = glade_xml_get_widget (xml, "pref-dialog");
  gtk_window_set_icon_from_stock (GTK_WINDOW (dialog), GTK_STOCK_PREFERENCES);

  g_signal_connect (dialog, "delete_event", G_CALLBACK (gtk_true), NULL);
  g_signal_connect_after (dialog, "response", G_CALLBACK (gtk_widget_hide), NULL);
  g_signal_connect_swapped (dialog, "destroy", G_CALLBACK (g_object_unref), client);

  widget = glade_xml_get_widget (xml, "pref-audio-combo");
  for (i = 2; i < ogmdvd_nlanguages; i++)
  {
    str = g_strdup_printf ("%s (%s)", ogmdvd_languages[i][OGMDVD_LANGUAGE_NAME], 
        ogmdvd_languages[i][OGMDVD_LANGUAGE_ISO639_1]);
    gtk_combo_box_append_text (GTK_COMBO_BOX (widget), str);
    g_free (str);
  }

  widget = glade_xml_get_widget (xml, "pref-subp-combo");
  for (i = 2; i < ogmdvd_nlanguages; i++)
  {
    str = g_strdup_printf ("%s (%s)", ogmdvd_languages[i][OGMDVD_LANGUAGE_NAME], 
        ogmdvd_languages[i][OGMDVD_LANGUAGE_ISO639_1]);
    gtk_combo_box_append_text (GTK_COMBO_BOX (widget), str);
    g_free (str);
  }

#if HAVE_MKV_SUPPORT
  widget = glade_xml_get_widget (xml, "container-combo");
  gtk_combo_box_append_text (GTK_COMBO_BOX (widget), _("Matroska Media (MKV)"));
#endif

#ifdef HAVE_X264_SUPPORT
  widget = glade_xml_get_widget (xml, "x264-radio");
  gtk_widget_set_sensitive (widget, TRUE);
#endif

#ifdef HAVE_THEORA_SUPPORT
  widget = glade_xml_get_widget (xml, "theora-radio");
  gtk_widget_set_sensitive (widget, TRUE);
#endif

#if defined(HAVE_SRT_SUPPORT) && defined(HAVE_ENCHANT_SUPPORT)
  widget = glade_xml_get_widget (xml, "spell-check");
  gtk_widget_set_sensitive (widget, TRUE);
#endif

  ogmrip_pref_init (xml, client, pref_conf);

  gconf_client_notify_add (client, OGMRIP_GCONF_OUTPUT_DIR, 
      (GConfClientNotifyFunc) ogmrip_pref_output_dir_notified, pref_conf[0].widget, NULL, NULL);

  str = gconf_client_get_string_default (client, OGMRIP_GCONF_OUTPUT_DIR, OGMRIP_DEFAULT_OUTPUT_DIR);
  gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (pref_conf[0].widget), str);
  g_free (str);

  gconf_client_notify_add (client, OGMRIP_GCONF_TMP_DIR, 
      (GConfClientNotifyFunc) ogmrip_pref_tmp_dir_notified, pref_conf[1].widget, NULL, NULL);

  str = gconf_client_get_string_default (client, OGMRIP_GCONF_TMP_DIR, OGMRIP_DEFAULT_TMP_DIR);
  gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (pref_conf[1].widget), str);
  ogmrip_set_tmp_dir (str);
  g_free (str);

  g_object_unref (xml);

  return dialog;
}

